Package: pcaPP
Version: 2.0-5
VersionNote: Released 2.0-4-1 on 2024-08-09 on CRAN
Title: Robust PCA by Projection Pursuit
Maintainer: Valentin Todorov <valentin.todorov@chello.at>
Authors@R: c(person(given = "Peter", family = "Filzmoser", role = "aut"),
              person(given = "Heinrich", family = "Fritz", role = "aut"),
              person(given = "Klaudius", family = "Kalcher", role = "aut"),
              person(given = "Valentin", family = "Todorov", role = "cre",
                     email = "valentin.todorov@chello.at"))
Depends: R(>= 3.6.2)
Imports: mvtnorm, methods
Suggests: robustbase
Description: Provides functions for robust PCA by projection pursuit.
    The methods are described in Croux et al. (2006) <doi:10.2139/ssrn.968376>,
    Croux et al. (2013) <doi:10.1080/00401706.2012.727746>,
    Todorov and Filzmoser (2013) <doi:10.1007/978-3-642-33042-1_31>.
License: GPL (>= 3)
URL: https://github.com/valentint/pcaPP
BugReports: https://github.com/valentint/pcaPP/issues
NeedsCompilation: yes
Packaged: 2024-08-19 10:48:56 UTC; valen
Author: Peter Filzmoser [aut],
  Heinrich Fritz [aut],
  Klaudius Kalcher [aut],
  Valentin Todorov [cre]
Repository: CRAN
Date/Publication: 2024-08-19 13:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:14:09 UTC; unix
Archs: pcaPP.so.dSYM

Package: pcaPP
Version: 2.0-5
VersionNote: Released 2.0-4-1 on 2024-08-09 on CRAN
Title: Robust PCA by Projection Pursuit
Maintainer: Valentin Todorov <valentin.todorov@chello.at>
Authors@R: c(person(given = "Peter", family = "Filzmoser", role = "aut"),
              person(given = "Heinrich", family = "Fritz", role = "aut"),
              person(given = "Klaudius", family = "Kalcher", role = "aut"),
              person(given = "Valentin", family = "Todorov", role = "cre",
                     email = "valentin.todorov@chello.at"))
Depends: R(>= 3.6.2)
Imports: mvtnorm, methods
Suggests: robustbase
Description: Provides functions for robust PCA by projection pursuit.
    The methods are described in Croux et al. (2006) <doi:10.2139/ssrn.968376>,
    Croux et al. (2013) <doi:10.1080/00401706.2012.727746>,
    Todorov and Filzmoser (2013) <doi:10.1007/978-3-642-33042-1_31>.
License: GPL (>= 3)
URL: https://github.com/valentint/pcaPP
BugReports: https://github.com/valentint/pcaPP/issues
NeedsCompilation: yes
Packaged: 2024-08-19 10:48:56 UTC; valen
Author: Peter Filzmoser [aut],
  Heinrich Fritz [aut],
  Klaudius Kalcher [aut],
  Valentin Todorov [cre]
Repository: CRAN
Date/Publication: 2024-08-19 13:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:51:14 UTC; unix
Archs: pcaPP.so.dSYM

useDynLib(pcaPP, .registration = TRUE)

importFrom(mvtnorm, rmvnorm)

importFrom("grDevices", "gray", "rainbow")
importFrom("graphics", "abline", "lines", "mtext", "par", "plot",
         "plot.new", "plot.window", "rect", "text", "title")
importFrom("stats", "cov", "cov2cor", "mad", "median", "qchisq",
         "qnorm", "rnorm", "runif", "sd")
importFrom("utils", "flush.console", "installed.packages")
importFrom("methods", "is")

export(PCAgrid, PCAproj, PCdiagplot, ScaleAdv, cor.fk, covPC,
       covPCAgrid, covPCAproj, data.Zou, l1median, l1median_BFGS,
       l1median_CG, l1median_HoCr, l1median_NLM, l1median_NM,
       l1median_VaZh, objplot, opt.BIC, opt.TPO, plotcov, qn, sPCAgrid)

S3method (plot, opt.TPO)
S3method (plot, opt.BIC)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `pcaPP`: Robust PCA by Projection Pursuit

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![R-CMD-check](https://github.com/valentint/pcaPP/workflows/R-CMD-check/badge.svg)](https://github.com/valentint/pcaPP/actions)
[![downloads](https://cranlogs.r-pkg.org/badges/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![license](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
<!-- badges: end -->

## Installation

The `pcaPP` package is on CRAN (The Comprehensive R Archive Network) and
the latest release can be easily installed using the command

    install.packages("pcaPP")

## Building from source

To install the latest stable development version from GitHub, you can
pull this repository and install it using

    ## install.packages("remotes")
    remotes::install_github("valentint/pcaPP")

Of course, if you have already installed `remotes`, you can skip the
first line (I have commented it out).

## Community guidelines

### Report issues and request features

If you experience any bugs or issues or if you have any suggestions for
additional features, please submit an issue via the
[*Issues*](https://github.com/valentint/pcaPP/issues) tab of this
repository. Please have a look at existing issues first to see if your
problem or feature request has already been discussed.

### Contribute to the package

If you want to contribute to the package, you can fork this repository
and create a pull request after implementing the desired functionality.

### Ask for help

If you need help using the package, or if you are interested in
collaborations related to this project, please get in touch with the
package maintainer.

####################################
##	Tradeoff Area - Optimization  ##
####################################

.TPO <- function (f.x = .l0sparse, f.y = .sumVar, obj.pc.0, obj.pc.1, fa.Name = "TPO", ...)
{
	if (missing (obj.pc.0))
		return (list (x = f.x (...), y = f.y (...)))
	if (missing (obj.pc.1))
		return (list (x = f.x (v.0 = obj.pc.0[[1]], ...), y = f.y (v.0 = obj.pc.0[[1]], ...)))

	x <- f.x (v.0 = obj.pc.0$x, v.1 = obj.pc.1$x, ...)
	y <- f.y (v.0 = obj.pc.0$y, v.1 = obj.pc.1$y, ...)

	return (-x * y)
}

.l1sparse <- function (x, pc, k, fa.Name = expression (paste (L[1], " Sparseness")), ...)
{
	p <- nrow (pc$load)
	n.k <- length (k)

	n.k * sqrt (p) - sum (abs (pc$loadings[,k]))
}

.l0sparse <- function (x, pc, k, zero.tol = 1e-16, fa.Name = expression (paste (L[0], " Sparseness")), ...)
{
	sum (abs (pc$loadings[,k]) <= zero.tol)
}

.pl1sparse <- function (x, pc, k, fa.Name = expression (paste (L[1], " Sparseness (%)")), ...)
{
	p <- nrow (pc$load)
	n.k <- length (k)

	.l1sparse (x, pc, k, ...) / ((sqrt (p) - 1) * n.k) * 100
}

.pl0sparse <- function (x, pc, k, zero.tol = 1e-16, fa.Name = expression (paste (L[0], " Sparseness (%)")), ...)
{
	p <- nrow (pc$load)
	n.k <- length (k)

	.l0sparse (x, pc, k, ...) / ((p - 1) * n.k) * 100
}


.lambda <- function (x, pc, k, fa.Name = expression (lambda), ...)
{
	if (length (k) == 1)
		return (pc$lambda[k])

	if (any (diff (pc$lambda) != 0) && length (k) > 1)
		stop ("This option can only be used when all considered PCs were calculated with the same lambda.")

	return (pc$lambda[1])
}

.vn <- function (n, k) 
{
	if (length (k) == 1)
		return (n)
	return (paste ("Cumulated", n))
}

.sumVar <- function (x, pc, k, NAME = FALSE, ...)
{
	if (NAME)
		return (.vn ("Explained Variance", k))
			
	return (sum (pc$sdev[k]^2))
}

.sumVarDiff <- function (x, pc, k, v.1 = 0, NAME = FALSE, ...)
{
	if (NAME)
		return (.vn ("diff Explained Variance", k))

	return (sum (pc$sdev[k]^2) - v.1)
}

.sumVarP <- function (x, pc, k, v.0, NAME = FALSE, ...)
{
	if (NAME)
		return (.vn ("Explained Variance (%)", k))

	if (missing (v.0))			##	return the total variance of x 
		return (sum (apply (x, 2, .getScaleFunction (pc$args$method))^2))

	return (sum (pc$sdev[k]^2) / v.0 * 100)
}

.sumVarDiffP <- function (x, pc, k, v.0, v.1 = 0, NAME = FALSE, ...)
{
	if (NAME)
		return (.vn ("diff Explained Variance (%)", k))

	if (missing (v.0))			##	return the total variance of x 
		return (sum (apply (x, 2, .getScaleFunction (pc$args$method))^2))

	return (sum (pc$sdev[k]^2) / v.0 * 100  - v.1)
}

.obj <- function (pc, fa.Name = "Objective Function", ...)
{
	pc$ev
}

#####################
##	BIC criterion  ##
#####################

.BIC.RSS <- function (obj.pc.0, f.BIC = .nBIC.Guo, f.RSS = .Calc.RSS.a, fa.Name = "BIC", ...)
#.BIC.RSS <- function (obj.pc.0, f.BIC = .nBIC.CC, f.RSS = .Calc.RSS.a, NAME = FALSE, ...)
#BIC.RSS <- function (obj.pc.0, f.BIC = .nBIC.CC, f.RSS = .Calc.RSS.e, get.name = FALSE, ...)
{
#	if (NAME)
#		return (paste (.GetFunctionName (f.BIC, ...), "(", .GetFunctionName (f.RSS, ...),")"))

	RSS <- f.RSS (...)

	if (missing (obj.pc.0))
		return (RSS)

	df_k <- .Calc.df_k (...)

	f.BIC (RSS0 = RSS, RSS = obj.pc.0, df_k = df_k, ...)
}

.Calc.df_k <- function (pc, k, zero.tol = 1e-15, ...) {	sum (abs (pc$loadings[,k]) > zero.tol)}

.Calc.RSS.e <- function (pc, k, fa.Name = "RSS: exact", ...)
{
	p <- nrow (pc$load)
	if (length (pc$sdev) < p)
		stop ("all eigenvectors needed for exact computation of RSS")

	sum (pc$sdev[-(1:max(k))]^2)	
}

.Calc.RSS.a <- function (x, pc, k, fa.Name = "RSS: approx", ...)
{
	if (ncol (pc$loadings) < length (k))
		stop ("at least k eigenvectors needed for approximate calculation of RSS")
	l <- pc$loadings[,1:max(k)]
	f.method <- .getScaleFunction (pc$args$method)
	sum (apply (x - x %*% l %*% t(l), 2, f.method)^2)
}

.nBIC.CC <- function (x, RSS0, RSS, df_k, k, fa.Name = "BIC", trace = 0, ...)
{
##	RSS0 as the restricted model's RSS
##	RSS as the unrestricted model's RSS
##	df_k as the number of non-zero loadings of the restricted model

	n <- nrow (x)
	BIC <- RSS0 / RSS + df_k * log (n) / n
	if (trace >= 2)
		cat ("RSS0:", RSS0, "; RSS:", RSS, "; df_k:", df_k, "; n:", n, "; BIC:", BIC, "\n")
	BIC	
}

.nBIC.Guo <- function (x, RSS0, RSS, df_k, k, fa.Name = "BIC", trace = 0, ...)
{
##	RSS0 as the restricted model's RSS
##	RSS as the unrestricted model's RSS
##	df_k as the number of non-zero loadings of the restricted model

	n <- nrow (x)
	BIC <- RSS0 / RSS + df_k * log (n)# / n
	if (trace >= 2)
		cat ("RSS0:", RSS0, "; RSS:", RSS, "; df_k:", df_k, "; n:", n, "; BIC:", BIC, "\n")
	BIC	
}

PCAproj <- function (x, k = 2, method = c ("mad", "sd", "qn"), CalcMethod = c("eachobs", "lincomb", "sphere"), nmax = 1000, update = TRUE, scores = TRUE, maxit = 5, maxhalf = 5, scale = NULL, center = l1median_NLM, zero.tol = 1e-16, control)
{
	if (!missing (control))
		###ParseControlStructure (control, c("k", "method", "CalcMethod", "nmax", "update", "scores", "maxit", "maxhalf"))
		.ParseControlStructure (control, c("k", "method", "CalcMethod", "nmax", "update", "scores", "maxit", "maxhalf", "center", "scale", "zero.tol"))

	method <- .getScaleMethod (method)

	CalcMethod <- match.arg (CalcMethod[1], c("eachobs", "lincomb", "sphere"))

	x <- .Conv2Matrix (x, substitute (x))

	n = nrow (x)
	p = ncol (x)

	if( k > min(n,p))
		stop ('k too large')

	if(p > n)
	{
			svdx = svd(t(x))
			x = svdx$v %*% diag(svdx$d)
			pold=p
			p=n
	} else
			pold=p

	DataObj = ScaleAdv (x, scale = scale, center = center)

    if (pold > n) # center and scale must have original data dimension:
	{
		DataObj$center <- as.vector(svdx$u%*%DataObj$center)
		DataObj$scale <- ScaleAdv(x%*%t(svdx$u),center=NULL,scale=scale)$scale
	}

	y = DataObj$x

#	m = l1median(x)
#	y = t(t(x) - m)

	if (scores)
		scoresize <- n * k
	else
		scoresize <- 0

	if (CalcMethod == "lincomb")
	{
		update = FALSE
		if (nmax > n)
		{
			aux = matrix (runif ((nmax-n) * n), nrow = nmax-n)
			##y = rbind (y, t(t(aux %*% x) - DataObj$center))
			y <- rbind (y, aux %*% y) ## use this instead?
		}
	}
	else if (CalcMethod == "sphere")
	{
		update = FALSE
		if(nmax  >n)
			#y[(n+1):nmax,] =	rmvnorm(nmax-n, rep(0,p), diag (p))
			y = rbind (y,		rmvnorm(nmax-n, rep(0,p), diag (p)))
	}

	nn = nrow (y)

	if (update)
		ret.C = .C (C_pcaProj_up,
				as.integer (c(nn, p, n, k, method, scores, maxit, maxhalf)),
				as.double (zero.tol),
				as.double (y),
				scores = double (scoresize),
				loadings = double (p * k),
				lambda = double (k))
	else
		ret.C = .C (C_pcaProj,
				as.integer (c(nn, p, n, k, method, scores)),
				as.double (zero.tol),
				as.double (y),
				scores = double (scoresize),
				loadings = double (p * k),
				lambda = double (k))

	veig = matrix (ret.C$loadings, ncol = k)

	idx.mo <- ret.C$lambda == -1
	if (any (idx.mo))
	{
		veig [, idx.mo] <- .Null (veig[, !idx.mo])
		ret.C$lambda[idx.mo] <- 0
	}

   if(pold>n)
		veig = svdx$u %*% veig

	if (scores)
		.DataPostProc (DataObj, ret.C$lambda, veig, matrix (ret.C$scores, ncol = k) , match.call(), scores)
	else
		.DataPostProc (DataObj, ret.C$lambda, veig, NULL, match.call(), scores)
}

.Null <- function (M)	##	 Null function from package MASS -> 2do: move calls to LAPACK to C++ code
{
    tmp <- qr(M)
    set <- if (tmp$rank == 0L)
        1L:ncol(M)
    else -(1L:tmp$rank)
    qr.Q(tmp, complete = TRUE)[, set, drop = FALSE]
}

.DataPostProc <- function (DataObj, obj, loadings, scores, cl, bScores)
{
	idx <- order (obj, decreasing = TRUE)
	obj <- obj [idx]
	loadings <- loadings [,idx, drop = FALSE]

	if (bScores)
		scores <- scores [,idx, drop = FALSE]

	ret <- list()

   ##loadings
	{
		c <- ncol (loadings)
		r <- nrow (loadings)
		ret$loadings <- loadings

		ret$loadings <- .loadSgnU (ret$loadings)

		if (is.null (dimnames (DataObj$x)[[2]]))
			dimnames (ret$loadings) <- list (paste (rep ("V", r), 1:r, sep = ""), paste (rep ("Comp.", c), 1:c, sep = ""))
		else
			dimnames (ret$loadings) <- list (dimnames (DataObj$x)[[2]], paste (rep ("Comp.", c), 1:c, sep = ""))

		class (ret$loadings) <- "loadings"
	}

   ##sdev
	ret$sdev <- as.numeric (obj)
	names (ret$sdev) <- dimnames (ret$loadings)[[2]]

   ##center
	ret$center <- DataObj$center
   ##scale
	ret$scale <- DataObj$scale
   ##n.obs
	ret$n.obs <- nrow (DataObj$x)

   ##scores
	if (bScores)
	{
		ret$scores <- scores
		dimnames (ret$scores) <- list (1:nrow (scores), dimnames (ret$loadings)[[2]]) ;
	}
	else
		ret$scores <- NULL

	ret$call <- cl

	class (ret) <- c ("pcaPP", "princomp")
	return (ret)
}

"PCdiagplot" <- function(x, PCobj, crit=c(0.975,0.99,0.999), ksel=NULL, plot=TRUE, plotbw=TRUE, raw=FALSE, colgrid="black", ...){
#
#  raw ... if raw==TRUE the raw SDist is computed, without using the median correction
#
# ksel ... select range for values of k
# colgrid ... color of the grid

if (is.null(PCobj$scores)) stop("No PC scores have been computed! Provide scores!")

n <- nrow(PCobj$sco)
k <- ncol(PCobj$loadings)
if (is.null(ksel)) {ksel <- seq(1,k)}
kl <- length(ksel)
if (ksel[kl]>k) {
	ksel <- seq(1,k)
	warning("Not so many PCs available as specified -> set to number of available PCs!")
}

# initialize output matrices
SDist <- matrix(NA,nrow=n,ncol=kl)
ODist <- matrix(NA,nrow=n,ncol=kl)
critOD <- matrix(NA,kl,length(crit))
critSD <- matrix(NA,kl,length(crit))

for (k in 1:kl){
  # compute score distances:
  if (ksel[k]==1){
    SDist[,k]=abs(PCobj$scores[,1])/PCobj$sdev[1]
  }
  else {
    SDist[,k]=sqrt(apply(t(t(PCobj$scores[,1:ksel[k]]^2)/PCobj$sdev[1:ksel[k]]^2),1,sum))
  }
  if (!raw){
    SDist[,k]=SDist[,k]*sqrt(qchisq(0.5,ksel[k]))/median(SDist[,k])
  }

  # compute orthogonal distances:
  xc <- scale(x,center=PCobj$center,scale=PCobj$scale)
  ODist[,k]=sqrt(apply((xc-PCobj$scores[,1:ksel[k]]%*%t(PCobj$loadings[,1:ksel[k]]))^2,1,sum))

  # compute critical values
  critSD[k,]=sqrt(qchisq(crit,ksel[k]))
  critOD[k,]=(median(ODist[,k]^(2/3))+mad(ODist[,k]^(2/3))*qnorm(crit))^(3/2)
}

# plot results:
if (plot)
{

    ## VT::10.10.2016 - remove a require(graphics) command
    ##
    ## require(graphics)
    if(plotbw)  pg = rev(gray(seq(0,0.7,length=ncol(critSD))))     # gray level to plot
    else        pg = rev(rainbow(ncol(critSD),start=0.9,end=0.1))  # color level to plot

    par(mfrow=c(1,2))

    plot(0,0,xlim=c(1,n),ylim=c(min(ksel),max(ksel)),type="n",xlab="Index of the observation",
       ylab="Number of PCs", ...)

    title("Orthogonal distance")
    for (i in 1:n){
        for (j in 1:length(ksel)){
            howbig=sum(ODist[i,j]>critOD[j,])
            if (howbig==0) { rect(i-0.5,ksel[j]-0.5,i+0.5,ksel[j]+0.5,border=colgrid) }
            else { rect(i-0.5,ksel[j]-0.5,i+0.5,ksel[j]+0.5,col=pg[howbig]) }
        }
    }

plot(0,0,xlim=c(1,n),ylim=c(min(ksel),max(ksel)),type="n",xlab="Index of the observation",
       ylab="Number of PCs", ...)
title("Score distance")
for (i in 1:n){
  for (j in 1:length(ksel)){
    howbig=sum(SDist[i,j]>critSD[j,])
    if (howbig==0) { rect(i-0.5,ksel[j]-0.5,i+0.5,ksel[j]+0.5,border=colgrid) }
    else { rect(i-0.5,ksel[j]-0.5,i+0.5,ksel[j]+0.5,col=pg[howbig]) }
  }
}

}
list(ODist=ODist,SDist=SDist,critOD=critOD,critSD=critSD)
}

qn <- function (x, corrFact)
{
	if (missing (corrFact))	
		corrFact = 1 / (sqrt(2) * qnorm(5/8))

	ret.C <- .C (C_qn,
			as.integer (length (x)),
			as.double (corrFact),
			parOutD = double (1),
			x = as.double (x))
	ret.C$parOutD
}

plot.opt.TPO <- function (x, k, f.x = c ("l0", "pl0", "l1", "pl1", "lambda"), f.y = c ("var", "pvar"), ...)
{
	.plot.sPCAgrid.opt.ind (x = x, k = k, f.x = f.x, f.y = f.y, ...)
}

plot.opt.BIC <- function (x, k, f.x = c ("l0", "pl0", "l1", "pl1", "lambda"), f.y = c ("var", "pvar"), ...)
{
	.plot.sPCAgrid.opt.tot (x = x, k = k, f.x = f.x, f.y = f.y, ...)
}

.eval.fx <- function (f.x)
{
	if (is.function (f.x))
		return (f.x)
	f.x <- f.x[[1]]

	.valf <- c ("l0", "l1", "lambda", "pl0", "pl1")

	f.x <- match.arg (f.x, .valf)
	f.x <- match (f.x, .valf)

	.F <- c (.l0sparse, .l1sparse, .lambda, .pl0sparse, .pl1sparse)

	return (.F[[f.x]])
}

.eval.fy <- function (f.y) #, supr)
{
	if (is.function (f.y))
		return (f.y)
	f.y <- f.y[[1]]

	.valf <- c("var", "dvar", "pvar", "dpvar") #, "obj") 

	f.y <- match.arg (f.y, .valf)
#	if (!missing (supr))
#		if (any (supr == f.y))
#			stop (paste ("Option \"", f.y, "\"cannot be used in this context", sep = ""))
	f.y <- match (f.y, .valf)

	.F <- c (.sumVar, .sumVarDiff, .sumVarP, .sumVarDiffP) #, .obj)

	return (.F[[f.y]])
}


.format.PCidx <- function (k, kto)
{
	if (length (k) > 1)
	{
		if (!missing (kto))
			stop ("if kto is specified, k must be of length 1")
		kto <- max (k)
		k <- min (k)
	}
	else
	{
		if (missing (kto))
			kto <- k
		else if (kto < k)
		{
			temp <- k
			k <- kto
			kto <- temp
		}
	}

	if (k == kto)
		return (paste ("PC", k))
	return (paste ("PCs ", k, "-", kto, sep = ""))
}

.title.trdoffC <- function (k, kto)
{
	paste ("Tradeoff Curve (", .format.PCidx (k, kto), ")", sep = "")
}

.plot.sPCAgrid.opt.tot <- function (x, k, main, ...) #, f.x = "l0", f.y = "pvar", ylim, xlab, ylab, main.pre, main.suf)
{
	if (missing (k))
		k = x$k.ini + length (x$pc)

	opt.idx <- k - x$k.ini

	stopifnot (length (k) == 1)
	stopifnot (any (class (x) == "sPCAgrid.opt.tot"))

	if (is.null(x$opt))
		stop ("x does not contain iteration data. Set \x22store.PCs = TRUE\x22 for function sPCAgrid.opt.tot")

	if (missing (main))
		main <- .title.trdoffC (x$k.ini + 1, k)

	ret <- .plot.opt (x = x$x, opt = x$opt, opt.idx = opt.idx, main = main, ...)
#	.opt.summary (x = x$x, pc = x$pc.noord[[opt.idx]], opt = x$opt, k = x$opt$k[[opt.idx]], ...)
	.opt.summary (x = x$x, opt = x$opt, opt.idx = opt.idx, idx.lambda = FALSE, ...)

	return (invisible (ret))
}

.plot.sPCAgrid.opt.ind <- function (x, k, main, ...) #, f.x = "l0", f.y = "pvar", ylim, xlab, ylab, main, main.pre, main.suf, ...)
{
	if (missing (k))
		k <- x$k.ini + 1

	stopifnot (length (k) == 1)
	stopifnot (any (class (x) == "sPCAgrid.opt.ind"))
	if (is.null(x$opt))
		stop ("x does not contain iteration data. Set \x22store.PCs = TRUE\x22 for function sPCAgrid.opt.ind")

	n.k <- length (x$pc$lambda)
	.assureRange (k, x$k.ini + 1, x$k.ini + n.k)

	opt.idx <- k - x$k.ini

	if (missing (main))
		main <- .title.trdoffC (k)

	opt <- x$opt[[opt.idx]]

	ret <- .plot.opt (x = x$x, opt = opt, main = main, ...)

	.opt.summary (x = x$x, pc = x$pc.noord, opt = opt, k = k, ...)

	return (invisible (ret))
}

.applyPFunc <- function (PCs, pc, idx.pc, f, ...)
{
	if (missing (pc))
		pc <- PCs[[idx.pc]]

	f.0 <- f (pc = PCs[[1]], ...)
	f.1 <- f (pc = PCs[[length (PCs)]], v.0 = f.0, ...)

	f (pc = pc, v.0 = f.0, v.1 = f.1, ...)
}

.plot.opt <- function (x, opt, opt.idx = 1, f.x = "pl0", f.y = "pvar", ylim, xlab, ylab, main, main.pre, main.suf, ...)
{
	k <- opt$k[[opt.idx]]

	f.x <- .eval.fx (f.x)
	f.y <- .eval.fy (f.y)

	if (missing (xlab))
		xlab <- .GetFunctionName (f.x, k = k)
	if (missing (ylab))
		ylab <- .GetFunctionName (f.y, k = k)

	PCs <- opt$PCs

	pdat <- .trdoff.sPCAgrid.ind (x, PCs, k = k, f.x, f.y, ...)

	if (missing (main))
		main <- paste ("Tradeoff Curve PC", k)
	if (!missing (main.pre))
		main <- paste (main.pre, main)
	if (!missing (main.suf))
		main <- paste (main, main.suf)

	if (missing (ylim))
		ylim <- c (0, max (pdat [,2]))
	plot (pdat[order(pdat[,1]),], type = "b", xlab= xlab, ylab = ylab, ylim = ylim, main = main)

	if (is.null (opt$idx.best))
		return (invisible (pdat))

	abline (v = pdat[opt$idx.best[opt.idx], 1], lty = 2)

	return (invisible (pdat))
}

.opt.summary <- function (x, pc, optmode, k, opt, opt.idx, idx.lambda = TRUE, ...)
{
	if (missing (pc))
		pc <- opt$pc[[opt.idx]]
	if (missing (k))
		k <- opt$k[[opt.idx]]
	if (missing (optmode))
		optmode <- opt$mode

	lam <- round (pc$lambda [max (k)], 2)

	if (length (k) > 1)
		k.use <- k
	else
		k.use <- 1:k

	vartot <- .sumVarP (x = x, pc = pc, k = k.use, ...)
	ecv <- .sumVarP  (x = x, pc = pc, k = k.use, v.0 = vartot, ...)
#	ecv <- .applyPFunc (opt$PCs, pc, f = .sumVarP, k = k.use, x = x, ...)
	ecv <- round (ecv, 2)
	
	LoS <- .l0sparse (pc = pc,k = k.use, ...)

#	LoS <- .applyPFunc (opt$PCs, pc, f = .l0sparse, k = k.use, x = x, ...)

	if (length (k.use) > 1)
		txtK <- paste (min(k.use), "-", max(k.use), sep = "")
	else
		txtK <- k.use

	txtECV <- bquote (paste ("ECV" [.(txtK)], ": ", .(ecv), "%"))
	txtLoS <- bquote (paste ("L"[0], "S" [.(txtK)], ": ", .(LoS)))


	if (idx.lambda)
	{
		ssL <- paste ("opt", max (k), sep = "")
		txtLambda <- bquote (paste (lambda[.(ssL)], ": " ,.(lam)))
	}
	else
		txtLambda <- bquote (paste (lambda[opt], ": " ,.(lam)))

	txt <- bquote(paste (.(optmode), " - ", .(txtLambda), "; ", .(txtECV), "; ", .(txtLoS)))

	mtext (txt, line = 0.25, cex = 0.8)
}

.trdoff.sPCAgrid.ind <- function (x, PCs, f.x, f.y, ...)
{
	n <- length (PCs)

	x.0 <- f.x (x = x, pc = PCs[[1]], ...)
	x.1 <- f.x (x = x, pc = PCs[[n]], v.0 = x.0, ...)

	y.0 <- f.y (x = x, pc = PCs[[1]], ...)
	y.1 <- f.y (x = x, pc = PCs[[n]], v.0 = y.0, ...)

	rx <- sapply (PCs, .flexapply, f = f.x, NAME = "pc", args = list (x = x, v.0 = x.0, v.1 = x.1, ...))
	ry <- sapply (PCs, .flexapply, f = f.y, NAME = "pc", args = list (x = x, v.0 = y.0, v.1 = y.1, ...))

	stopifnot (is.numeric (rx))
	stopifnot (is.numeric (ry))

	ret <- cbind (rx, ry)
#	ret <- ret [order (ret[,1]), ]

	class (ret) <- "trdoff.sPCAgrid"
	ret 
}

.assureRange <- function (x, l, u, name = substitute (x))
{
	if (missing (l))
	{
		if (x > u)
			stop (paste (name, "must not be larger than ", u))
	}
	else if (missing (u))
	{
		if (x < l)
			stop (paste (name, "must not be smaller than ", l))
	}
	else if (x < l || x > u)
		stop (paste (name, "must be between", l, "and", u))
}

objplot <- function (x, k, ...)
{
	if (any (class (x) == "sPCAgrid.opt.tot"))
	{
		if (missing (k))
			k = x$k.ini + length (x$pc)

		.assureRange (k, x$k.ini, x$k.ini + length (x$pc))

		opt.idx <- k - x$k.ini
		.objplot (x$opt, opt.idx = opt.idx)
		.opt.summary (x = x$x, opt = x$opt, opt.idx = opt.idx, idx.lambda = FALSE, ...)
	}
	else if (any (class (x) == "sPCAgrid.opt.ind"))
	{
		if (missing (k))
			k = x$k.ini + 1
		.assureRange (k, x$k.ini, x$k.ini + length (x$pc$lambda))

		opt <- x$opt[[k - x$k.ini]]
		.objplot (opt, k = k)
		.opt.summary (x = x$x, pc = x$pc.noord, opt = opt, k = k, ...)
	}
}

.objplot <- function (opt, opt.idx = 1, k, main, xlab, ylab, type = "b", ...)
{
	obj <- opt$obj[,opt.idx]

	if (missing (k))
		k <- opt$k[[opt.idx]]

	lambda <- sapply (opt$PCs, get ("[["), "lambda")
	if (is.matrix (lambda))
		lambda <- lambda [nrow (lambda),]

	if (missing (main))
		if (length (k) > 1)
			main <- paste ("Model Selection (PCs ", min (k), "-", max(k), ")", sep = "")
		else
			main <- paste ("Model Selection (PC ", k, ")", sep = "")

	if (missing (xlab))
		xlab <- expression (lambda)
	if (missing (ylab))
		ylab <- paste ("Objective Function (", opt$mode, ")", sep = "")

	plot (lambda, obj, xlab = xlab, ylab = ylab, main = main, type = type, ...)

	cl <- lambda [which.min (obj)]

	abline (v = cl, lty = 2)

#	txt <- bquote (lambda[opt] == .(round (cl, 2)))
#	mtext (line = 0.25, text = txt, cex = 0.8)

	invisible ()
}

opt.TPO <- function (x, k.max = ncol (x), n.lambda = 30, lambda.max, ...)
{
	store.opt = TRUE
	ret <- .sPCAgrid.opt.ind (x = x, k.max = k.max, n.lambda = n.lambda, lambda.max = lambda.max, store.PCs = store.opt, f.eval = .TPO, ...)
	class (ret) <- c (class (ret), "opt.TPO")
	return (ret)
}

opt.BIC <- function (x, k.max = ncol (x), n.lambda = 30, lambda.max, ...)
{
	store.opt = TRUE
	ret <- .sPCAgrid.opt.tot (x = x, k.max = k.max, n.lambda = n.lambda, lambda.max = lambda.max, store.PCs = store.opt, f.eval = .BIC.RSS, ...)
	class (ret) <- c (class (ret), "opt.BIC")
	return (ret)
}

.flexapply <- function (X, f, NAME, args)
{
	args[[NAME]] <- X
	do.call (f, args)
}

.sPCAgrid.ml <- function (..., lambda, f.pca = .sPCAgrid.ini, f.apply = lapply)
{
	args <- list (...)
	args$store.call <- FALSE
	f.apply (X = lambda, FUN = .flexapply, f = f.pca, NAME = "lambda", args = args)
}

#.listset <- function (x, y, name = "y")
#{
#	stopifnot (length (x) == length (y))
#
#	for (i in 1:length (x))
#		x[[i]][[name]] <- y[[i]]
#	x
# }

.sPCAgrid.opt.tot <- function (x, n.lambda = 101, k.max = 2, lambda, lambda.ini, lambda.max, trace = 0, store.PCs = TRUE, f.apply = lapply, f.eval = .TPO, ...)
{
	pc.ini	<- NULL
	f.pca <- .sPCAgrid.ini
#	f.eval <- .TPO

	if (!missing (lambda.ini) && !is.null (lambda.ini))			##	use lambda.ini
	{
		k.ini <- length (lambda.ini)
		pc.ini <- f.pca (x = x, lambda = lambda.ini, k = k.ini, cut.pc = FALSE, ...)

		if (!missing (pc.ini) && !is.null (pc.ini))
			warning ("argumens \x22pc.ini\x22 AND \x22lambda.ini\x22 were specified. Ignoring \x22pc.ini\x22.")
	}
	else if (!missing (pc.ini) && !is.null (pc.ini))			##	use pc.ini
	{
		k.ini = pc.ini$k
		lambda.ini <- rep (NA, k.ini)
	}
	else 
	{
		pc.ini <- NULL
		lambda.ini <- NULL
		k.ini <- 0
	}

#	if (missing (lambda) && !missing (lambda.max))								##	Change 20120407
#		lambda.max <- rep (lambda.max, len = k.max)								##	Change 20120407

	p <- ncol (x)
	if (k.ini == p)
		stop ("all components have already been computed")

	if (k.ini + k.max > p)
	{
		warning (paste ("reducing k.max to", p - k.ini))
		k.max <- p - k.ini
	}

	if (missing (lambda))
	{
#		if (missing (lambda.max) || is.na (lambda.max[i]))						##	Change 20120407
		if (missing (lambda.max) || is.na (lambda.max))
			max.fs <- .FSgetLambda (x = x, k = k.max, pc.ini = pc.ini, f.pca = f.pca, scores = FALSE, trace = trace, ...)
		else
#			max.fs <- lambda.max[i]												##	Change 20120407
			max.fs <- lambda.max[1]

		lambda <- seq (0, max.fs, len = n.lambda)
	}

	PCs <- .sPCAgrid.ml (x = x, pc.ini = pc.ini, k = k.max, scores = FALSE, cut.pc = TRUE, trace = trace, f.pca = f.pca, lambda = lambda, f.apply = f.apply, ...)

	opt <- .SPCAgrid.opt (x, PCs, f.eval, store.PCs, k = k.ini + 1:k.max, singlePC = FALSE, ...)

	if (!store.PCs)
		return (opt)

	ret <- list (pc = list (), pc.noord = list (), 
					x = x, k.ini = k.ini, opt = opt)

	for (i in 1:length (opt$pc))
	{
		pc <- .cut.pc (opt$pc[[i]], k.ini + k.max)
		ret$pc.noord[[i]] <- pc
		ret$pc[[i]] <- .orderPCs (pc, k.max, k.ini, TRUE)
	}

	class (ret) <- "sPCAgrid.opt.tot"
	return (ret)
}

.sPCAgrid.opt.eval <- function (x, f.eval = .BIC.RSS, k = 1, ...)
{
	if (all (class (x) != "sPCAgrid.opt.tot"))
		stop ("x must be of type \"sPCAgrid.opt.tot\"")
	##2do:  check parameter k

	.SPCAgrid.opt (x$x, x$opt$PCs, f.eval, storePCs = FALSE, k = 1:k, ...)
}

.SPCAgrid.opt <- function (x, PCs, k, f.eval, store.PCs, f.apply = sapply, singlePC = TRUE, ...)
{
	ret <- list ()

	if (store.PCs)
		ret$PCs <- PCs

	if (missing (f.eval) || !is.function (f.eval))
	{
		if (!store.PCs)
			stop ("either store.PCs must be TRUE, or f.eval must be a valid model evaluation function")
		return (ret)
	}

	ret$pc <- ret$k <- list ()

	ret$mode <- .GetFunctionName (f.eval, ...)

	for (i in 1:length (k))
	{
		if (singlePC)
			K <- k[i]
		else
			K <- k[1:i]

		obj.pc.0 <- f.eval (x = x, pc = PCs[[1]], k = K, ...)
		#.l0sparse (x = x, px = PCs[[1]], k = K, ...)
		obj.pc.1 <- f.eval (x = x, pc = PCs[[length (PCs)]], k = K, obj.pc.0 = obj.pc.0, ...)
		#.sumVar (x = x, pc = PCs[[length (PCs)]], k = K, ...)

		obj <- f.apply (X = PCs, FUN = .flexapply, f = f.eval, NAME = "pc", args = list (x = x, k = K, obj.pc.0 = obj.pc.0, obj.pc.1 = obj.pc.1, ...))
		ret$obj <- cbind (ret$obj, obj)

		idx.best <- which.min (obj)
		ret$idx.best <- cbind (ret$idx.best, idx.best)
		ret$pc[[i]] <- PCs[[idx.best]]
		ret$k[[i]] <- K
	}

	if (!store.PCs)
		return (ret$pc)

	return (ret)
}

.sPCAgrid.opt.ind <- function (x, n.lambda = 101, k.max = ncol (x), lambda.ini, lambda.max, trace = 0, store.PCs = TRUE, f.eval = .TPO, ...)
{
	pc.ini	<- NULL
	f.pca <- .sPCAgrid.ini

	if (!missing (lambda.ini) && !is.null (lambda.ini))		##	go for lambda.ini
	{
		k.ini <- length (lambda.ini)
		pc.ini <- f.pca (x = x, lambda = lambda.ini, k = k.ini, cut.pc = FALSE, ...)
		if (!missing (pc.ini))
			stop ("either store.PCs must be TRUE, or f.eval must be a valid model evaluation function")
	}
	else if (!missing (pc.ini) && !is.null (pc.ini))		##	go for pc.ini
	{
		k.ini = pc.ini$k
		lambda.ini <- rep (NA, k.ini)
	}
	else 
	{
		pc.ini <- NULL
		lambda.ini <- NULL
		k.ini <- 0
	}

	if (!missing (lambda.max))
		lambda.max <- rep (lambda.max, len = k.max)

	p <- ncol (x)
	if (k.ini == p)
		stop ("all components have already been computed")

	if (k.ini + k.max > p)
	{
		warning (paste ("reducing k.max to", p - k.ini))
		k.max <- p - k.ini
	}

	if (store.PCs)
		opt <- list ()

	for (i in 1:k.max)
	{
		if (missing (lambda.max) || is.na (lambda.max[i]))
			max.fs <- .FSgetLambda (x = x, k = 1, pc.ini = pc.ini, f.pca = f.pca, scores = FALSE, trace = trace, ...)
		else
			max.fs <- lambda.max[i]

		lambda <- seq (0, max.fs, len = n.lambda)

		cur.pcs <- .sPCAgrid.ml (x = x, pc.ini = pc.ini, k = 1, scores = FALSE, cut.pc = FALSE, trace = trace, lambda = lambda, f.pca = f.pca, ...)

		cur.opt <- .SPCAgrid.opt (x, cur.pcs, f.eval, store.PCs, k = k.ini + i, ...)

		if (store.PCs)
			opt[[i]] <- cur.opt

		pc.ini <- cur.opt$pc[[1]]
	}

	pc.ini <- .cut.pc (pc.ini, k.ini + k.max)
	pc <- .orderPCs (pc.ini, k.max, k.ini, TRUE)

	if (!store.PCs)
		return (pc)

	ret <- list (pc = pc, pc.noord = pc.ini, x = x, k.ini = k.ini, opt = opt)

	class (ret) <- "sPCAgrid.opt.ind"

	return (ret)
}

.GFSL.calc.sPCA <- function (k.check, lambda = 1, f.pca = sPCAgrid, zero.tol = 1e-10, trace = 0, check.all = TRUE, ...)
{
	if (trace >= 5)
		.flush.cat ("checking lambda: ", lambda, "\r\n", sep = "")

	if (check.all)
		return (f.pca (lambda = lambda, k = k.check, ...)$loadings[, 1:k.check])

	return (f.pca (lambda = lambda, ...)$loadings[, k.check, drop = FALSE])
}

.GFSL.is.sparse <- function (k.check, zero.tol = 1e-10, ...)
{
	load <- .GFSL.calc.sPCA (k.check = k.check, zero.tol = zero.tol, ...)
	return ((sum (abs (load)> zero.tol) ) == ncol (load))
}

.GFSL.is.same <- function (sparse.load, zero.tol = 1e-10, ...)
{
	load <- .GFSL.calc.sPCA (zero.tol = zero.tol, ...)

	return (sum (abs (sparse.load - load)) <= zero.tol)
}

.GFSL.find.max <- function (lambda = 1, ...)
{
	for (i in 1:16)
	{
		if (.GFSL.is.sparse (lambda = lambda, ...))
			return (lambda)

		lambda = lambda * 2
	}
	return (NULL)	
}

.GFSL.find.range <- function (lbL , ubL, niter = 6, f.sparse = .GFSL.is.sparse, ...)
{
	for (i in 1:niter)
	{
		mbL <- mean (c(ubL, lbL))
		if (f.sparse (lambda = mbL, ...))
			ubL <- mbL
		else
			lbL <- mbL
	}
	return (ubL)
}

.getFullSparseLambda.all <- function (uBound, ...)
{	
	if (missing (uBound))
		uBound <- .GFSL.find.max (lambda = 1, check.all = TRUE, ...)

	if (is.null (uBound))
		stop ("cannot find full sparse model")

	lBound <- ifelse (uBound > 1, uBound / 2, 0)

	.GFSL.find.range (lbL = lBound, ubL = uBound, f.sparse = .GFSL.is.sparse, ...)
}

.getFullSparseLambda.indiv <- function (niter = 15, ...)
{
	uBound <- .GFSL.find.max (lambda = 1, check.all = FALSE, ...)

	if (!is.null (uBound))
	{
#		return (.getFullSparseLambda.all (uBound = uBound, ...))
		lBound <- ifelse (uBound > 1, uBound / 2, 0)

		return (.GFSL.find.range (lbL = lBound, ubL = uBound, f.sparse = .GFSL.is.sparse, check.all = FALSE, ...))
	}

	lambda.max <- 1e6

	load <- .GFSL.calc.sPCA (lambda = lambda.max, check.all = FALSE, ...)

	.GFSL.find.range (lbL = 0, ubL = lambda.max, niter = 25, f.sparse = .GFSL.is.same, sparse.load = load, check.all = FALSE, ...)
}



.FSgetLambda <- function (...)
{							##	find lambda which yields full sparseness

							##	Problem: if due to pc.ini the full sparse loadings matrix (vector)
							##		is not a 0-1 matrix (vector):
							##		In such a case full sparseness is achvieved, if a change of lambda 
							##		wouldn't change the loadings matrix (vector) any more
	kc <- .FSgetK.check (...)
	if (.FSpossible (...))
		.FSgetLambdaFS (..., k.check = kc)
	else
		.FSgetLambdaC (..., k.check = kc)
}

.FSgetK.check <- function (pc.ini = NULL, k.ini, k, ...)
{
	if (is.null (pc.ini))
		return (1:k)

	if (missing (k.ini))
		k.ini <- pc.ini$k

	return (k.ini + 1 : k)
}

.FSpossible <- function (pc.ini = NULL, k.ini, k, zero.tol = 1e-16, ...)
{
	if (is.null (pc.ini))
		return (TRUE)

	if (missing (k.ini))
		k.ini <- pc.ini$k

	l0 <- abs (pc.ini$loadings[, 1:k.ini, drop = FALSE]) > zero.tol
							##	return whether at least one row of lambda does only have zeros.
	return (any (rowSums (l0) == 0))
							
}

.FSgetLambdaFS <-function (iter = 15, ...) 
{
	.FSiter (..., f.test = .FSisFullSparse)
}

.FSgetLambdaC <- function (zero.tol, ...)
{
	pc0 <- .FScalc (..., lambda = 2^20)[[1]]
	.FSiter (..., pc0 = pc0, f.test = .FScompL)
}

.FSiter <- function (niter = 8, testL = testE^-2, testU = testE^5, testE = 16, ...)
{
	L <- testL
	U <- testU

	for (i in 1:2)
	{
		lambda.test <- testE^seq (log (L) / log (testE), log (U) / log (testE), len = niter)
		sparse <- .FStest (..., lambda = lambda.test)

		if (sparse[1])
			return (lambda.test[1])
		if (!sparse[niter])
			return (lambda.test[niter])
		idx <- which (sparse)[1]

		L <- lambda.test[idx-1]
		U <- lambda.test[idx]
	}
	return (lambda.test[idx])
}

.FStest <- function (..., f.test)
{
	##	2do. check, wether f.apply is specified. if not, do a binary search!
	PCs <- .FScalc (...)
	sapply (PCs, f.test, ...)
}

.FScompL <- function (pc, pc0, k.check, zero.tol = 1e-16, ...)
{
	sum (abs (.loadSgnU (pc$load [, k.check, drop = FALSE]) - .loadSgnU (pc0$load [, k.check, drop = FALSE])) > sqrt (zero.tol)) == 0
}

.FScalc <- function (...)	{	.sPCAgrid.ml (...)	}

.FSisFullSparse <- function (pc, k.check, zero.tol = 1e-16, ...)
{
	n.k <- length(k.check)									##	number of loadings vectors to check
	sum (abs (pc$load [, k.check]) > sqrt (zero.tol)) == n.k		##	the number of non-zero loadings is equal to n.k
}

.loadSgnU <- function (x)
{														##	change the signs of the columns of a loadings matrix such, that each column's absolute maximum is positive
	idx.max <- apply (abs (x), 2, which.max)
	sgn <- sign (x[cbind (idx.max, 1:ncol (x))])
	if (length (sgn) == 1)
		return (x * sgn)
	return (x %*% diag (sgn))
}

l1median <- function (X, MaxStep = 200, ItTol = 10^-8, trace = 0, m.init = .colMedians (X))
{
	if (is.null (dim(X)))
		return (median (X))
	l1median_NLM (X = X, maxit = MaxStep, tol = ItTol, trace = trace, m.init = m.init)$par
}

#l1median = function (X, MaxStep = 200, ItTol = 10^-8, trace = 0)
#{
#	if (trace >= 0)
#		warning ("This function (pcaPP::l1median)is outdated.\r\nFor better performance try any of pcaPP::l1median_* instead. Preferably pcaPP::l1median_NLM.\r\nOtherwise use (trace = -1) for suppressing this warning. ")
#
#	if (class (X) != "matrix")
#	{
#		if (class (X) == "data.frame")
#			X = as.matrix(X)
#		else
#			X = matrix(X, ncol = 1)
#	}
#
#	ret = .C ("l1median", PACKAGE="pcaPP",
#		as.double (X),
#		as.integer (nrow(X)),
#		as.integer (ncol(X)),
#		med = double (ncol(X)),
#		ret = integer(1),
#		as.integer (MaxStep),
#		as.double (ItTol)
#		)
#
#	if (ret$ret != 0)
#		return (ret$med)
#	stop("iteration failed")
#}


l1median_BFGS <-
function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), REPORT = 10, ...)
{
	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))
		
		
	ret = .C (C_l1median_BFGS, NAOK = TRUE,
		par = as.integer (c(dim (X), maxit, trace, REPORT)),
		npar.out = integer (4),
		dpar = as.double (c(-Inf, tol)),
		dpar.out = double (1),
		as.double (X),
		#as.double (pscale),
		med = as.double (m.init)#double (ncol(X))
		)

	if (trace >= 1)
		cat ("l1median returned", ret$npa, ret$dpar, "\r\n") ;

	return (list (par = ret$med, value = ret$dpar.out[1], code = ret$npar.out [1], iterations = ret$npar.out [2], iterations_gr = ret$npar.out [3], time = ret$npar.out[4]))
}


l1median_CG <-
function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), ...)#, type = 1)
{
	type = 1
	
	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))
		
	if (type < 1 || type > 3)
		stop ("parameter type MUST be either 1, 2 or 3")
		
	ret = .C (C_l1median_CG, NAOK = TRUE,
		par = as.integer (c(dim (X), maxit, trace, type)),
		npar.out = integer (4),
		dpar = as.double (c(-Inf, tol)),
		dpar.out = double (1),
		as.double (X),
		#as.double (pscale),
		med = as.double (m.init)#double (ncol(X))
		)

	if (trace >= 1)
		cat ("l1median returned", ret$npa, ret$dpar, "\r\n") ;

	return (list (par = ret$med, value = ret$dpar.out[1], code = ret$npar.out [1], iterations = ret$npar.out [2], iterations_gr = ret$npar.out [3], time = ret$npar.out[4]))
}


l1median_HoCr <-
function (X, maxit = 200, tol = 10^-8, zero.tol = 1e-15, trace = 0, m.init = .colMedians (X), ...)
{
	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))

	ret.C = .C (C_l1median_HoCr,
        npar = as.integer (c(dim (X), maxit, trace)),
		npar.out = integer (4),
		as.double (c (tol, zero.tol)),
		as.double (X),
		med = as.double (m.init))

	if (trace >= 1)
	{
		if (ret.C$npar.out[1] == 1)
			cat ("Algorithm did not converge (return code 1).\n")
		else if (ret.C$npar.out[1] == 2)
			cat ("Step halving failed (return code 2).\n")
		else if (ret.C$npar.out[1] == 3)
			cat ("A concentration of more than n/2 observations in one point has been detected (return code 3).\n")
	}


	return (list (par = ret.C$med, value = 	sum (sqrt (colSums ((t(X) - ret.C$med)^2))),
		 code = ret.C$npar.out[1], iterations = ret.C$npar.out [2] + 1, time = ret.C$npar.out[3]))
}

l1median_NLM <-
function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), ...)
{
	msg = 8
	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))

	ret = .C (C_l1median_NLM, NAOK = TRUE,
		npar = as.integer (c(dim (X), maxit, 0, 0, 0, msg, trace)),
		dpar = as.double (c(tol, 0)),
		as.double (X),
		med = as.double (m.init)
		)

	if (trace >= 1)
		cat ("l1median returned", ret$npa, ret$dpar, "\r\n") ;
		
		if (ret$npar[7])
			stop (paste ("nlm optimization returned error code", ret$npar[7]))

	return (list (par = ret$med, value = ret$dpar[2], code = ret$npar[4], iterations = ret$npar [3], time = ret$npar[6]))
}


.l1median_NLM_Hess <-
function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), msg = 8, method = 1, GFlag = 1, HFlag = 1, Exp = 1, Digits = 6, ...)
{
	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))

	ret = .C (C_l1median_NLM_Hess, NAOK = TRUE,
		npar = as.integer (c(dim (X), maxit, 0, method, 0, msg, trace, GFlag, HFlag, Exp, Digits)),
		dpar = as.double (c(tol, 0)),
		as.double (X),
		med = as.double (m.init)
		)

	if (trace >= 1)
		cat ("l1median returned", ret$npa, ret$dpar, "\r\n") ;

		if (ret$npar[7])
			stop (paste ("nlm optimization returned error code", ret$npar[7]))

	return (list (par = ret$med, value = ret$dpar[2], code = ret$npar[4], iterations = ret$npar [3], time = ret$npar[6]))
}


l1median_NM <-
#function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), alpha = 1, beta = 0.5, gamma = 2, ...)
function (X, maxit = 200, tol = 10^-8, trace = 0, m.init = .colMedians (X), ...)
{
	alpha = 1
	beta = 0.5
	gamma = 2
	
	X <- .Conv2Matrix (X, substitute (X))
	
	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))
		
	ret = .C (C_l1median_NM, NAOK = TRUE,
		npar = as.integer (c(dim (X), maxit, 0, 0, 0, 0, trace)),
		dpar = as.double (c(-Inf, tol, 0, alpha, beta, gamma)),
		as.double (X),
		#as.double (pscale),
		med = as.double (m.init)#double (ncol(X))
		)

	if (trace >= 1)
		cat ("l1median returned", ret$npa, ret$dpar, "\r\n") ;

	return (list (par = ret$med, value = ret$dpar[3], code = ret$npar[4], iterations = ret$npar [6], time = ret$npar[7]))
}

l1median_VaZh <-
function (X, maxit = 200, tol = 10^-8, zero.tol = 1e-15, trace = 0, m.init = .colMedians (X), ...)
{

	X <- .Conv2Matrix (X, substitute (X))

	if (length (m.init) != ncol (X))
		stop (paste ("length of vector m.init (=", length (m.init), ") does not match the number of columns of data object X (=", ncol (X),")", sep = ""))

	ret.C = .C (C_l1Median_VZ,
        npar = as.integer (c(dim (X), maxit, 0, trace)),
		nParOut = integer (3),
		as.double (c (tol, zero.tol)),
		as.double (X),
		med = as.double (m.init))

	return (list (par = ret.C$med, value = sum (sqrt (colSums ((t(X) - ret.C$med)^2))),
		 code = ret.C$nParOut[1], iterations = ret.C$nParOut [2], time = ret.C$nParOut[3]))
}

#cor.fk <- function (x, y = NULL, cor = TRUE)					#corissue
cor.fk <- function (x, y = NULL)
{
	cor = TRUE													#corissue
	if (is.null (y))
	{		##	x is expected to be matrix or data frame
		if (!is.matrix (x) && ! is.data.frame (x))
			stop ("x must be either numeric vector, matrix or data.frame.")
		{
			p <- ncol (x)
			dn <- colnames (x)
			ret <- diag (p)
			dimnames (ret) <- list (dn, dn)

			for (i in 1:p)
			{
				if (!cor)	##	calculating the diagonal elements. if cor == TRUE the diagonal elements are all equal to 1 and hence don't need to be calculated
#					ret[i, i] <- cor.fk (x[, i], x[, i], cor = cor)
					ret[i, i] <- cor.fk (x[, i], x[, i])	#corissue

				if (i == p)
					return (ret)

				ord <- order (x[, i])
				cur.x <- x[ord, i]
				for (j in (i+1):p)
					ret[i, j] <- ret[j, i] <- .cor.fk.2d (cur.x, x[ord,j], cor)
			}
		}
	}
	else
	{
		if (length (x) != length (y))
			stop ("x and y must have same length.")
		ord <- order (x)
		return (.cor.fk.2d (x[ord], y[ord], cor))
	}
}

.cor.fk.2d <- function (x, y, cor)
{
	if (length (x) != length (y))
		stop ("x and y must have same length.")

	ret <- .C (C_kendallNlogN, NAOK = FALSE, DUP = TRUE,		##	20130322 set DUP = TRUE - C_kendallNlogN implementation modifies x & y vectors!!
				as.double (x),
				as.double (y),
				as.integer (c (length (x), cor)),
				ret = double (1))
	return (ret$ret)
}

## this will render the output independent from the version of the package
suppressPackageStartupMessages(library(pcaPP))

##  Sparse loadings
set.seed (0)
##x <- data.Zou()
data(iris)
x <- iris[, 1:4]

l1median_NM (x)$par
l1median_CG (x)$par
l1median_BFGS (x)$par
l1median_NLM (x)$par
l1median_HoCr (x)$par
l1median_VaZh (x)$par

# compare with coordinate-wise median:
apply(x,2,median)

pc <- PCAgrid(x)
pc
summary(pc)
pc$loadings
pc$scores

plotcov <- function (cov1, cov2, method1, labels1, method2, labels2, ndigits = 4, ...)
{

	if (class (cov1) == "matrix")
		cm1 = cov1
	else if (is.null (cov1$cov))
		stop ("No appropriate covariance structure specified")
	else
	{
		cm1 = cov1$cov
		if (!is.null (cov1$method))
			method1 = cov1$method
	}

	if (missing (method1)) #|| is.null(method1))
		method1 = "Method 1"

	if (ncol (cm1) != nrow (cm1))
		stop ("Supplied covariance structure has to be quadratic!")

	if (missing (labels1))
		if (is.null (dimnames (cm1)))
			labels1 = paste ("V", 1:ncol (cm1), sep = "")
		else
			labels1 = dimnames (cm1)[[1]]

	if (missing (cov2))
	{	# only plotting cov1
		.plotsingle (cm1, labels1, method1, ndigits, ...)
	}
	else
	{
		if (class (cov2) == "matrix")
			cm2 = cov2
		else if (is.null (cov2$cov))
			stop ("No appropriate covariance structure specified")
		else
		{
			cm2 = cov2$cov
			if (!is.null (cov2$method))
				method2 = cov2$method
		}
		if (missing (method2))# || is.null(method2))
			method2 = "Method 2"

		if (ncol (cm2) != nrow (cm2))
			stop ("Supplied covariance structure has to be quadratic!")

		if (missing (labels2))
			if (is.null (dimnames (cm2)))
				labels2 = paste ("V", 1:ncol (cm2), sep = "")
			else
				labels2 = dimnames (cm2)[[1]]

		.plotcomp (cm1, cm2, labels1, labels2, method1, method2, ndigits, col = "#0096FF", ...)
	}
	invisible()
}

.plotcomp <- function (cor1, cor2, labels1, labels2, method1, method2, ndigits = 4, ...)	## internal function
{
	plot.new()
# 	plot (0,0, pch = "", axes = F, xlab = "", ylab = "")
	oldmar = par ("mar")
	par (mar = rep(1, 4))
	p = ncol (cor1)
	lim = c(-1, p + 1)
	plot.window(xlim=lim, ylim= lim, xaxs="i", yaxs="i")

	for (i in 1:p)
	{
		text (i - 0.5, p + 0.5, labels1[i], srt=90)
		text (-0.5, p - i + 0.5, labels2[i])
	}

	for (i in 2:p)
	{
		for (j in 1:(i-1))
		{
			.doEllipses (cor1[c(i,j), c(i,j)], pos = c(i - 1.5,p - j - 0.5), lwd = 2)
			.doEllipses (cor2[c(i,j), c(i,j)], pos = c(i - 1.5, p - j - 0.5), lwd = 2, ...)

			text (j - 0.5,p - i + 0.5, round (cor1[i,j], ndigits), adj = c(0.5,-0.1))
			text (j - 0.5,p - i + 0.5, round (cor2[i,j], ndigits), adj = c(0.5,1.1), ...)
		}
	}

	lines (c(0.5, p-0.5), c(p - 0.5, 0.5), lwd = 3)

#	lines (mean (lim) + 

	lines (lim[2] - 2 + c(-0.5, 0.3), c(-0.3, -0.3))
	lines (lim[2] - 2 + c(-0.5, 0.3), c(-0.7, -0.7), ...)
	text (lim[2] - 2 - 0.7, -0.275, method1, pos = 2)
	text (lim[2] - 2 - 0.7, -0.675, method2, pos = 2)

	par (mar = oldmar)
}


.plotsingle <- function (cm, labels, method, ndigits, ...)
{
	plot.new()

	oldmar = par ("mar")

	par (mar = rep(1, 4))
	p = ncol (cm)
	lim = c(-1, p + 1)

	plot.window(xlim = lim, ylim = lim, xaxs = "i", yaxs = "i")

	for (i in 1:p)
	{
		text (i - 0.5, p + 0.5, labels[i], srt=90)
		text (-0.5, p - i + 0.5, labels[i])
	}

	for (i in 2:p)
	{
		for (j in 1:(i-1))
		{
			.doEllipses (cm[c(i,j), c(i,j)], pos = c(i - 1.5,p - j - 0.5), lwd = 2, ...)
			text (j - 0.5 ,p - i + 0.5, round (cm[i,j], ndigits), ...)
		}
	}

	lines (c(0.5, p-0.5), c(p - 0.5, 0.5), lwd = 3)

	lines (lim[2] - 2 + c(-0.5, 0.3), c(-0.5, -0.5), col = col, ...)
	text (lim[2] - 2 - 0.7, -0.475, method, pos = 2)

	par (mar = oldmar)
}


.doEllipses <- function (acov, pos, ...)		## internal function
{
	acov = cov2cor (acov)
	cov.svd <- svd(acov, nv = 0)
	r <- cov.svd[["u"]] %*% diag(sqrt(cov.svd[["d"]]))

	m <- 100

	alphamd <- c(1/3)

#	par(pty="s")

    e1md <- cos(c(0:m)/m * 2 * pi) * alphamd
    e2md <- sin(c(0:m)/m * 2 * pi) * alphamd
    emd <- cbind(e1md, e2md)
    ttmd <- t(r %*% t(emd)) + rep(1, m + 1)
    lines(ttmd[, 1] + pos[1], ttmd[, 2]+ pos[2], ...)
}

.sPCAgrid.R <- function(x, k = 1, splitcircle = 10, maxiter = 10, method = mad, lambda = 0, cut.pc = TRUE, trace =  0, f.rho = .ident, glo.scatter = 0, ord.mod = 2, center, scale, pc.ini, k.ini, inc.scores = FALSE, HDred = c ("eigen", "qr", "svd", "svd.mean", FALSE))
{

# grid search for PCA, using Householder transformation for orthogonality of loadings
#
# x ... centered (!) data matrix
# k ... number of components to determine
# splitcircle ... number of directions to search
# maxiter ... maximum number of iterations
# method ... how to estimate the standard deviation
#        should be: "sd", "mad", "qn"
#
	stopifnot (is.numeric (trace))
	
	
	n <- nrow (x)
	p <- ncol (x)
	
	pHD <- 0	
	HDProj <- NULL

	f.HDred <- c ("eigen", "qr", "svd.mean", "svd", FALSE)
	HDred <- pmatch (HDred[1], f.HDred)
	if (is.na (HDred))
		HDred <- 0
	else
		HDred <- f.HDred[HDred]

    if(p > n)			## Dimension reduction for high dimensional datatsets
    {
	    if (HDred == "svd")
	    {
	        svdx <- svd(t(x))
	        x <- svdx$v %*% diag (svdx$d)
	        HDProj <- svdx$u
    	}
	    else if (HDred == "svd.mean")
	    {
		    x.m <- colMeans (x)
	        svdx <- svd(t(x) - x.m)
	        x <- svdx$v[, -n] %*% diag (svdx$d[-n])
	        HDProj <- svdx$u[, -n]
    	}
	    else if (HDred == "qr")
	    {
			qrtx <- qr (t (x))
			x <- t (qr.R (qrtx))
			HDProj <- qr.Q (qrtx)
		}
		else if (HDred == "eigen")
		{
			e <- eigen (cov (x))
			ev <- e$vectors[,order (e$values, decreasing = TRUE)[1:(n-1)]]
			x <- x %*% ev
			HDProj <- ev[]
		}
		
		if (!is.null (HDProj))
		{
			p <- ncol (x)
			n <- nrow (x)
#cat ("reduced dimensions -> n x p =", n, "x", p, "\n")
		}
   	}

   	stopifnot (k >= 1)

	len.lambda = length (lambda)

	if (len.lambda != 1 && len.lambda != k)
		warning ("length (lambda) should either be equal to 1 or k")
	lambda <- rep (lambda, len = k)

	if (!missing (pc.ini) && !is.null (pc.ini))
	{
		if (missing (k.ini))
			k.ini <- pc.ini$k
	}
	else
		k.ini <- 0

	if (k.ini)
	{
		stopifnot (nrow (pc.ini$load) == ncol (x))
		stopifnot (ncol (pc.ini$load) == ncol (x))

		if (missing (k.ini))
			k.ini <- pc.ini$k
		stopifnot (k.ini + k <= ncol (x))

		k <- k.ini + k
		
		stopifnot (length (pc.ini$sdev) == ncol (x))

		sdev <- pc.ini$sdev

		l <- pc.ini$load
	}
	else
	{
		k.ini <- 0
		sdev <- rep (NA, p)
		l <- diag (p)
	}

	if (missing (center))
		center = NULL
	if (missing (scale))
		scale = NULL

	x <- ScaleAdv (x, center, scale)$x

	stopifnot (k <= ncol (x))

# 	if (splitcircle %% 2 == 0)				##	forcing splitcircle to be odd (-> obj. function always increases during maxiter - apart from numerical issues)...
# 	#if (splitcircle %% 2)					##	forcing splitcircle to be even
# 		splitcircle = splitcircle + 1

	if (k.ini)
		y <- x %*% l[,(k.ini + 1):p]
	else
		y <- x
		
	
	if (glo.scatter == 0)
		scl.mean <- sqrt (mean (apply (x, 2, method)^2))
	else
		scl.mean <- 1

	for (nb in (k.ini + 1):k)
	{  # loop over number of comp
		if (glo.scatter == 1)
			scl.mean <- sqrt (mean (apply (y, 2, method)^2))

		p1 <- p - nb + 1 # dimension will be reduced for subsequent PCs			##	p1 = dimensionality of remaining subspace

		if (p1 == 1)	##	if the subspace is one dimensional the only thing left to do is to calculate sdev of the remaining (nx1) matrix Y
		{				##	the loadings are already fixed - there's no free parametes (d.o.f) left
			sdev [nb] = method (y)
			next (nb)
		}

			# ordering of variables by variance

		nord <- order(apply(y,2,method),decreasing=TRUE)				##	get the decreasing order of the scale estimates.

		yord <- y[,nord]												##	order the data regarding to their scale..

			#	initializing the lincomb
		afin <- c(1, rep(0,p1-1)) 										##	with (1, 0, 0, 0, ...)
		yopt = yord[,1]													##	y %*% afin

			# outer loop to run until convergence

		for (i in 0:maxiter)											##	the 0 round initializes the system (as before), with div =2^i = 1
		{
			sumabsdelta = 0
			for (j in 1:p1)
			{
				if (abs (afin [j]) == 1)	##  if abs (afin[j]) == 1 -> only the jth variable is considered so far (norm (afin) == 1)-> 
					next (j)				##		it wouldn't change anything adding the jth component again. we would only run into numerical issues.

				rf = c(sqrt (1 - afin[j]^2), afin[j])	## rotation factors = cos (alpha), sin (alpha)

				if (afin [j] != 0)			##	 setting afin[j] to zero. only necessary when afin [j] != 0
				{							##		-> changing afin[-j] accordingly
					yopt <- (yopt - yord[,j] * rf[2]) / rf[1]
					afin [-j] <- afin [-j] / rf[1]
					afin [j] <- 0
				}

				cury <- cbind(yopt,yord[,j])
#				if (glo.scatter >= 3)
#					scl.mean <- sqrt (mean (apply (cury, 2, method)^2))

				res <- .gridplane.shrk(n = splitcircle, div = 2^i, curL = rf[2], y = cury, method = method, afin = afin, nord = nord, curP = j, curK = nb, l = l, lambda = lambda[nb - k.ini], f.rho = f.rho, scl.mean = scl.mean, u = HDProj)

				sumabsdelta = sumabsdelta + abs (res$alphamax[2] - rf[2])	##	sums up the absolute change of the angles

				rf <- res$alphamax			## rotation factors <- c (cos (alpha), sin (alpha))

				yopt <- yopt * rf[1] + yord[,j] * rf[2]
				afin[-j] = afin[-j] * rf[1]
				afin[j] = rf[2]

#cat ("loadings:", afin, "\n")
			}

			afin=afin/sqrt(sum(afin^2))		##	normieren. -> why?
			objf <- res$objmax

			{		##	thinking about dropping this block. if splitcircle odd -> the obj can't drop (or?!?!?)! (because the best candidate in the last round would hence be a candidate in this round too)
				if (!i || objf>=objfold)
				{
					objfbest <- objf
					objfold <- objf
					sclbest <- 	res$sclmax
					afinbest <- afin/sqrt(sum(afin^2)) ##  why again?
				}
				else if (trace >= 2)
					cat ("objective function dropped:", objfold - objf, "\r\n")
			}
			if (sumabsdelta < 1e-16)
			#if (2^-i < 1e-16)
			{
				cat ("stopping execution after", i + 1, "loops\n")
				break ## breaks the maxiter loop, as there's no more progress..
			}
		} # maxiter

		sdev[nb] <- sclbest

# 		if (length (afinbest) == 2)
# 		{
# 			afin[nord] <- afinbest
# 			afinNV <- matrix (c (afin, afin[2], -afin[1]), nrow = 2)
# 		}
# 		else
		{
		    afinNV <- diag(p1)
	
		    if (ord.mod == 1)
		    {
		    	afin <- afinbest				##	xxxord1
		    	idx.ref <- 1					##	xxxord1
	    	}
		    else
		    {
		    	afin[nord] <- afinbest			##	xxxord2
		    	idx.ref <- nord[1]#which.max (abs (afin))##	xxxord2
	    	}

		    N <- afinNV[,idx.ref]-afin

		    N.norm <- .norm (N)

		    if (N.norm > 1e-6)		##	use zero.tol
		    {
				if (afin [idx.ref] < 0)	##	we want the first (one particular - which?) value to be positive
					afin <- -afin
				N <- N / N.norm

				afinNV <- afinNV - (2*N %*% t(N))
		    }

			#afinNV <- cbind (afin, Null (afin))		##	Null creates the orthogonal complement of afin	    

			if (ord.mod == 1)
			{
				afinNV[nord,] <- afinNV		##	xxxord1
														##	nord must be inversed HERE, not before creating the backtransformation.
														##	otherwise the result would depend on the input order of variables (due to numerical issues when creating the back transformation matrix)
														##		thus ord.mod == 1 is the "better" way? why is default 2 then? maybe because the "real" PCAgrid algo does it that way?
			}
			else
				afinNV <- afinNV[,nord]			##	xxxord2
		}

		## afinNV contains afin + its orthogonal complement
		l[,nb:p] <- l [,nb:p] %*% afinNV

		y <- y %*% afinNV[,-1]						##	projecting the data into the complement of afin.

		if (trace >= 3)
		{
			y1=x%*%l[,(nb+1):p]
			cat ("numerical instability for component", nb, ":", .norm (y-y1), "\r\n")
		}
	} # loop over number of comp

	ord <- order(sdev,decreasing=TRUE)
	sdev <- sdev[ord]

	l <- l [,ord, drop = FALSE]
	
	proj.loadings <- NULL
	if(!is.null (HDProj))			##	undo Projection for high dimensional datasets
	{
#		proj.loadings <- l
		l <- HDProj %*% l
	}

	if (cut.pc)
	{
		l <- l [,1:k, drop = FALSE]
		sdev <- sdev [1:k]
	}

	ret <- list (sdev = sdev, loadings = l, scores = NULL, k = k, call = match.call) #, u = HDProj, proj.loadings = proj.loadings, x = x)

	if (inc.scores)
		ret$scores <- x %*% l

	ret	
}

.gridplane.new <- function (n = 10, div = 1, curL = 0, y, method, ...)
{
	nangle <- seq (-pi/2, pi/2, len = n) / div	+	##	the angles to check
					asin (curL)						##	the angle - component of the current loading
	alpha <- cbind(cos(nangle),sin(nangle))

	obj <- apply(y %*% t(alpha), 2, method)											##	VVV like gridplane VVV
	idx.max = which.max (obj)
	list(objmax=obj[idx.max],alphamax=alpha[idx.max,], sclmax=obj[idx.max], idx.max = idx.max)
}

.ident <- function (i) 1
.inverse <- function (i) 1/(i+1)

.gridplane.shrk <- function (n = 10, div = 1, curL = 0, y, method, afin, nord, curP, curK, l, lambda, f.rho = .ident, scl.mean, u, ...)
{
	nangle <- seq (-pi/2, pi/2, len = n) / div	+	##	the angles to check
					asin (curL)						##	the angle - component of the current loading
	alpha <- cbind(cos(nangle),sin(nangle))

													##	afinN the new linear combinations, one column for each angle tested...
	afinN <- afin %*% t(alpha[,1])						##	applying the effect of the new param "curP" to all other params
	afinN[curP,] <- alpha[,2]							##	setting the new param "curP"

	afinN[nord,] <- afinN							##	reversing the sort order of the variables

	p <- nrow (l)

	
	#nOrigParams <- l [, curK:p] %*% afinN			##	transforming the new params back into the "real" system of Coordinates

	if (!missing (u) && !is.null (u))				##	undo the svd transformation for high dimensional datasets
		u <- u %*% l [, curK:p]
	else
		u <- l [, curK:p]
	nOrigParams <- u %*% afinN

# cat ("backtrans: \n")
# print (u)
# cat ("\n\n")
	
	shrk <- colSums (abs (nOrigParams))				##	the shrinkage factors for each angle

	scl <- apply(y %*% t(alpha), 2, method)			##	the scale for each angle
	obj <- scl^2 - lambda * shrk * scl.mean^2 * f.rho (curK)		##	objective function...
#	obj <- - shrk
#browser ()	
	idx.max = which.max (obj)
	nang <- nangle[idx.max]
# 	if (nang < 0.4)
# 		nang = - (nang - pi / 2)

#cat ("checking angles ", paste (nangle, " (", - lambda * shrk * scl.mean^2, ", ", obj, "),", collapse = " ", sep = ""), "\n", sep = "")



#cat ("selected angle", format (nang, digits = 22), "\n")
	list(objmax = obj[idx.max], alphamax = alpha [idx.max,], sclmax = scl[idx.max], idx.max = idx.max)
}

.norm <- function (x) sqrt (sum(x^2))
.sumsq <- function (x) sum (x^2)

sPCAgrid <- function(x, k = 2, method = c ("mad", "sd", "qn"), lambda = 1#, norm.q = 1, norm.s = 1
					, maxiter = 10, splitcircle = 25, scores = TRUE, zero.tol = 1e-16, center = l1median, scale, trace = 0, store.call = TRUE, control, ...)
{

	norm.q <- .get_par (list (...), "norm.q", 1)
	norm.s <- .get_par (list (...), "norm.s", 1)
	glo.scatter <- .get_par (list (...), "glo.scatter", 0)

	check.orth <- FALSE
	dat <- list (x = x, substitute_x = substitute (x), k = k, method = method, maxiter = maxiter, splitcircle = splitcircle, check.orth = check.orth, scores = scores, lambda = lambda, zero.tol = zero.tol, center = center, store.call = store.call, trace = trace, ...)

	if (!missing (scale))	dat$scale <- scale

	if (!missing (control))
		dat <- .ParseControlStructureC (dat, control)

#	dat$check.orth <- FALSE		##	2do: remove this value!
	dat$HDred <- FALSE
	dat$cut.pc <- TRUE
    dat$glo.scatter <- glo.scatter
	dat$SpeedUp <- 0

	dat <- .sPCAgrid..DataPreProc (dat)
	dat$call <- match.call ()

	n <- nrow (dat$x)

	ret.C <- .C (C_sPCAgrid, NAOK = TRUE,
				nParIn = as.integer (c(dim (dat$x), dat$k, dat$splitcircle, dat$maxiter, dat$method, dat$trace, dat$k.ini, dat$check.orth, dat$glo.scatter, dat$pHD, dat$SpeedUp)),
				nParOut = integer (1),
				dParIn = as.double (c (dat$zero.tol, norm.q, norm.s)),
				as.double (dat$x),
				l = as.double (dat$l),
				sdev = as.double (dat$sdev),
				obj = double (dat$k),
				as.double (dat$lambda),			## length = k - k.ini + 1
				as.double (dat$HDProj))

	return (.sPCAgrid..PostProc (dat, ret.C))
}

PCAgrid <- function (x, k = 2, method = c ("mad", "sd", "qn"), maxiter = 10, splitcircle = 25, scores = TRUE, zero.tol = 1e-16, center = l1median, scale, trace = 0, store.call = TRUE, control, ...)
{
	check.orth <- FALSE
	dat <- list (x = x, substitute_x = substitute (x), k = k, method = method, maxiter = maxiter, splitcircle = splitcircle, check.orth = check.orth, scores = scores, zero.tol = zero.tol, center = center, store.call = store.call, trace = trace, ...)

	if (!missing (scale))	dat$scale	<- scale

	if (!missing (control))
		dat <- .ParseControlStructureC (dat, control)

#	dat$check.orth <- FALSE		##	2do: remove this value!
	dat$HDred <- TRUE
	dat$cut.pc <- TRUE
	dat$glo.scatter <- 0
	dat$SpeedUp <- 0

	dat <- .PCAgrid..DataPreProc (dat)
	dat$call <- match.call ()

	n <- nrow (dat$x)

	ret.C <- .C (C_PCAgrid, NAOK = TRUE,
				nParIn = as.integer (c(dim (dat$x), dat$k, dat$splitcircle, dat$maxiter, dat$method, dat$trace, dat$k.ini, dat$check.orth)),
				nParOut = integer (1),
				dParIn = as.double (dat$zero.tol),
				as.double (dat$x),
				l = as.double (dat$l),
				sdev = as.double (dat$sdev),
				obj = double (dat$k))

	return (.PCAgrid..PostProc (dat, ret.C))
}

.sPCAgrid.ini <- function	(x, k = 2, method = c ("mad", "sd", "qn"), norm.q = 1, norm.s = 1, maxiter = 10, splitcircle = 25, scores = TRUE, zero.tol = 1e-16, center = l1median, scale, store.call = TRUE, trace = 0, cut.pc = TRUE,
							pc.ini, k.ini, ord.all = FALSE, HDred = FALSE, lambda = 1, glo.scatter = 0, SpeedUp = 0, check.orth = FALSE, control, ...)
{
	dat <- list (x = x, substitute_x = substitute (x), k = k, method = method, maxiter = maxiter, splitcircle = splitcircle, scores = scores, lambda = lambda, zero.tol = zero.tol, center = center, store.call = store.call, trace = trace, cut.pc = cut.pc, glo.scatter = glo.scatter, ord.all = ord.all, HDred = HDred, SpeedUp = SpeedUp, check.orth = check.orth, ...)

	if (!missing (scale))	dat$scale <- scale
	if (!missing (pc.ini))	dat$pc.ini <- pc.ini
	if (!missing (k.ini))	dat$k.ini <- k.ini

	if (!missing (control))
		dat <- .ParseControlStructureC (dat, control)

	dat <- .sPCAgrid..DataPreProc (dat)
	dat$call <- match.call ()

	n <- nrow (dat$x)

	ret.C <- .C (C_sPCAgrid, NAOK = TRUE,
				nParIn = as.integer (c(dim (dat$x), dat$k, dat$splitcircle, dat$maxiter, dat$method, dat$trace, dat$k.ini, dat$check.orth, dat$glo.scatter, dat$pHD, dat$SpeedUp)),
				nParOut = integer (1),
				dParIn = as.double (c (dat$zero.tol, norm.q, norm.s)),
				as.double (dat$x),
				l = as.double (dat$l),
				sdev = as.double (dat$sdev),
				obj = double (dat$k),
#				max.maha = double (n),
				as.double (dat$lambda),			## length = k - k.ini + 1
				as.double (dat$HDProj))

	return (.sPCAgrid..PostProc (dat, ret.C))
}

.ParseControlStructureC <- function (x, control)
{
	if (is.null (control) ||
		!length (control))
		return (x)

	nc <- names (control)
	if (is.null (nc) ||
		any (nc == ""))
		stop ("Each item of list \x22control\x22 must have a name.")

	for (i in 1:length (control))
		x[[nc[i]]] <- control[[i]]
	return (x)
}

.sPCAgrid..DataPreProc <- function (x)
{
	len.lambda = length (x$lambda)

	if (len.lambda != 1 && len.lambda != x$k)
		warning ("length (lambda) should either be equal to 1 or k")
	x$lambda <- rep (x$lambda, len = x$k)

	x <- .PCAgrid..DataPreProc (x)

	return (x)
}

.validScaleMethods <- function () c ("sd", "mad", "qn")
.getScaleMethod <- function (x)
{
	.validMethods <- c ("sd", "mad", "qn")
	x <- x[1]
	if (is.character (x))
	{
		method <- match.arg (x, .validScaleMethods ())
		return (match (method, .validScaleMethods()) - 1)
	}

	if (x >= 3
		&& x != 5                                                               ## hack for the sPCAgrid-paper -> remove this line again...
		)
		stop ("the method is supposed to be a value < 3")

	return (x)
}

.getScaleName <- function (x)
{
	if (is.function (x))
		return (.GetFunctionName (x))

	return (.validScaleMethods () [.getScaleMethod (x) + 1])
}


.getScaleFunction <- function (x)	##	was .EvalScaleFunction before
{
	if (is.function (x))
		return (x)

	f.idx <- .getScaleMethod (x) + 1

	F <- c (sd, mad, qn)
	return (F [[f.idx]])
#	return  (eval (parse (text = method)))
}

.Check_DimRed <- function (x)
{
		##	dimension reduction
	x$pHD <- 0			##	stores the original p, if we have to reduce the data matrix' dimensionality (n < p)

	x$x.orig <- x$x

    if(x$p > x$n && x$HDred)			## Dimension reduction for high dimensional datatsets
    {
		svdx <- svd(t(x$x))
		x$svdx <- svdx

		x$x <- svdx$v %*% diag (svdx$d)
		x$HDProj <- svdx$u

		x$pHD <- x$p
		x$p <- ncol (x$x)
		x$n <- nrow (x$x)
		if (x$trace >= 2)
			cat ("reduced dimensions -> n x p =", x$n, "x", x$p, "\n")
   	}

	return (x)
}


.scale <- function(x)
{
	x$scl <- ScaleAdv (x$x, x$center, x$scale)
	x$x <- x$scl$x

	if (x$pHD) # center and scale must have original data dimension:
	{
		x$scl$center <- as.vector(x$svdx$u%*%x$scl$center)
		x$scl$scale <- ScaleAdv(x$x%*%t(x$svdx$u), center = NULL, scale = x$scale)$scale
	}

	return (x)
}


.Check_pc.ini <-function (x)
{
	if (!is.null (x$pc.ini))
	{
		if (is.null (x$k.ini))
			x$k.ini <- x$pc.ini$k
	}
	else
		x$k.ini <- 0

	if (x$k.ini)
	{
		stopifnot (all (dim (x$pc.ini$load) == ncol (x$x)))
		stopifnot (x$k.ini + x$k <= ncol (x$x))
		if (is.null (x$k.ini))
			x$k.ini <- x$pc.ini$k

		x$k <- x$k.ini + x$k	##	2do: don't change k anymore to k.ini + k
		stopifnot (length (x$pc.ini$sdev) == ncol (x$x))
		x$sdev <- x$pc.ini$sdev
		x$l <- x$pc.ini$load
	}
	else
	{
		x$k.ini <- 0
		x$sdev <- rep (NA, x$p)
		x$l <- diag (x$p)
	}

	return (x)
}

.PCAgrid..DataPreProc <- function (x)
{

	x$args <- list (method = x$method)		## 2do: which further args go here?
	##x[names (x) != "x"]	##	storing the function arguments

	x$x <- X <- .Conv2Matrix (x$x, x$substitute_x)

	if (is.function (x$method) && !is.null (x$method.name))
		attributes (x$method)$NAME <- x$method.name

	##	checking/initializing parameters

	x$n <- nrow (x$x)
	x$p <- ncol (x$x)

	stopifnot (is.numeric (x$trace))
	stopifnot (x$k >= 1)
	stopifnot (length (x$k) == 1)

	x$method <- .getScaleMethod (x$method)

	x <- .Check_DimRed (x)	#
	x <- .Check_pc.ini (x)

	stopifnot (x$k <= ncol (x$x))

	x <- .scale (x)

	return (x)
}

.sPCAgrid..GetLambda.ini <- function (x)
{
	if (!x$k.ini)
		return (NULL)
	if (is.null (x$pc.ini$lambda))
		return (rep (0, x$k.ini))
	return (x$pc.ini$lambda)
}

.sPCAgrid..PostProc <- function (x, ret.C)
{
	ret <- .PCAgrid..PostProc (x, ret.C)

	ret$lambda <- c (.sPCAgrid..GetLambda.ini (x), x$lambda)

	ret
}

.cut.pc <- function (ret, k)
{
	ret$loadings <- ret$loadings [, 1:k, drop = FALSE]
	ret$sdev <- ret$sdev [1:k]
	if (!is.null (ret$scores))
		ret$scores <- ret$scores[, 1:k, drop = FALSE]
	if (!is.null (ret$lambda))
		ret$lambda <- ret$lambda [1:k]

	return (ret)
}

.orderPCs <- function (ret, k, k.ini, ord.all = FALSE)
{
	if (is.null (ord.all) || ord.all)
		idx.ord <- 1:k
	else
		idx.ord <- (k.ini + 1):k

	if (length (idx.ord) == 1)	##	nothing to sort.
		return (ret)

	ord <- order(ret$sdev[idx.ord], decreasing = TRUE)
	ord <- idx.ord[ord]

	ret$pc.order <- 1:k
	ret$pc.order[idx.ord] <- ord

	ret$sdev[idx.ord]  <- ret$sdev[ord]
	ret$loadings[, idx.ord]  <- ret$loadings [, ord]
	ret$obj[idx.ord]  <- ret$obj [ord]

	if (!is.null (ret$lambda))
		ret$lambda[idx.ord]  <- ret$lambda [ord]


	return (ret)
}

.PCAgrid..PostProc <- function (x, ret.C)
{

	ret <- list (sdev = ret.C$sdev,
				loadings = matrix (ret.C$l, ncol = x$p, nrow = x$p), k = x$k,
				obj = ret.C$obj,
				n.obs = nrow (x$x.orig), args = x$args, #call = x$call,
				scale = x$scl$scale, center = x$scl$center
#				, max.maha = ret.C$max.maha
				)

	if (x$store.call)
		ret$call = x$call

	if(x$pHD)			##	undo SVD for high dimensional datasets
		ret$loadings <- x$HDProj %*% ret$loadings

	if (x$cut.pc)
		ret <- .cut.pc (ret, x$k)

	ret$loadings <- .loadSgnU (ret$loadings)

	ret <- .orderPCs (ret, x$k, x$k.ini, x$ord.all)

	ndn <- list (NULL, paste ("Comp", 1:ncol(ret$loadings), sep = "."))	##	new dimnames

	if (is.null (dimnames (x$x)[[2]]))
		ndn[[1]] <- paste ("X", 1:nrow(ret$loadings))
	else
		ndn[[1]] <- dimnames (x$x.orig)[[2]]

	dimnames (ret$loadings) <- ndn

	if (x$scores)
	{
		ret$scores <- t (t (x$x.orig) - x$scl$center) %*% ret$loadings
		dimnames (ret$scores) <- list (dimnames (x$x)[[1]], ndn[[2]])
	}

	class (ret$loadings) <- "loadings"
	class (ret) <- "princomp"

	return (ret)
}

.get_par <- function (l, idx, default)
{
	ret <- l[[idx]]
	if (is.null (ret))
		return (default)
	return (ret)
}

#.HDred <- function ()
#{
#	if (HDred == "svd")
#	{
#		svdx <- svd(t(x))
#		x <- svdx$v %*% diag (svdx$d)
#		HDProj <- svdx$u
#	}
#	else if (HDred == "svd.mean")
#	{
#	x.m <- colMeans (x)
#	svdx <- svd(t(x) - x.m)
#	x <- svdx$v[, -n] %*% diag (svdx$d[-n])
#	HDProj <- svdx$u[, -n]
#	}
#	else if (HDred == "qr")
#	{
#	qrtx <- qr (t (x))
#	x <- t (qr.R (qrtx))
#	HDProj <- qr.Q (qrtx)
#	}
#	else if (HDred == "eigen")
#	{
#	e <- eigen (cov (x))
#	ev <- e$vectors[,order (e$values, decreasing = TRUE)[1:(n-1)]]
#	x <- x %*% ev
#	HDProj <- ev
#	}
#
#	if (!is.null (HDProj))
#	{
#	pHD <- p
#	p <- ncol (x)
#	n <- nrow (x)
#	if (trace >= 2)
#		cat ("reduced dimensions -> n x p =", n, "x", p, "\n")
#	}
#}

.centr <- function(X,m) t (t(X) - m)


.mymean <- function (X)
{
	if (any (!is.na (X)))
		mean (X[!is.na (X)], trim = 0.4)
	else
		1
}

#.scale <- function (X)
#{
#	med <- apply (X, 2, median)
#	dev <- abs (.centr (X, med))
#	dev[dev == 0] <- NA	
#	apply(dev, 2, .mymean)
#}

# .First.lib <-
# function(lib,pkg)
# {	##	2do: delete, using namespaces now; this function is useless.
#    library.dynam("pcaPP",pkg,lib)
#    library(mvtnorm)
#    cat("pcaPP 0.1-1 loaded\n")
# }

.ParseControlStructure = function (control, arguments)
{
	if (!is.list(control))
		stop ("Invalid argument type: control structure must be of type list")

	if (missing(arguments))
		arguments = attributes(control)$names

	for (curname in arguments)
	{
		if (!is.null (control[[curname]]))	##	 if this argument is provided in the control structure:
			eval (parse (text = paste ("eval.parent (substitute(", curname, "<- control$", curname, "), n = 3)", sep = "")))
	}
}

.ParseDevString = function (method)
{
	if (method[1] == "mad")  return (0)
	if (method[1] == "sd")  return (1)
	if (method[1] == "Qn" | method[1] == "qn" )  return (2)
	return (1)
}

.Conv2Matrix <- function (x, sx = substitute (x))
{
	if(is.matrix(x))
		return (x)
	if(is.data.frame(x))
		return (data.matrix(x))
	return (matrix(x, nrow = length(x), ncol = 1, dimnames = list(names(x), deparse(sx))))
}


.colMedians <- function (x)
{
	if (is.null (dim (x)))
		return (median (x))
	apply (x, 2, median)
}

.GetFunctionName <- function (f, ...)
{
	form <- formals (f)
	if (!is.null (form$fa.Name))
		return (eval (form$fa.Name))

	if (!is.null (form$NAME))
		return (f (NAME = TRUE, ...))

	if (!is.null (attributes (f)$NAME))
		return (attributes (f)$NAME)

	return (NULL)
}

.flush.cat <- function (...)
{
	cat (...)
	flush.console ()	
}



.colSds <- function (x)	##	function for replacing sd until the R-people remove the sd matrix warning message
{
    if (is.data.frame(x)) 
        x <- as.matrix(x)

    if (!is.array (x) || length (dim (x)) != 2)
	stop ("'x' must be an array of two dimensions")

    apply (x, 2, sd)
}

covPC <- function (x, k = ncol (x$loadings), method)
{
	if (!any(class(x) == "princomp"))
		stop ("Invalid parameter \x22k\x22: Data type princomp expected!")
	if (length (x$sdev) != length (x$center))
		warning ("Calculating a rank", length (x$sdev), "- covariance matrix")

	ret = list()
	k = min (ncol (x$loadings), k)

	ret$cov = x$loadings[,1:k] %*% diag (x$sdev[1:k]^2) %*% t(x$loadings[,1:k])
	ret$center = x$center
	if (missing (method))
		ret$method = "Covariance estimation based on PCs"
	else
		ret$method = method

	class (ret) <- "covPC"

	return (ret)
} 

covPCAgrid <- function (x, control)
{
	pcs = PCAgrid (x, k = ncol(x), control = control)

	ret = list()
	ret$cov = pcs$loadings %*% diag (pcs$sdev^2) %*% t(pcs$loadings)
	ret$center = pcs$center
	ret$method = "Robust cov - estimation based on PCs (grid mode)"

	if (!missing (control) && !is.null (control$method))
		ret$method = paste ("Robust cov - estimation based on PCs (grid mode -", control$method, ")", sep = "")
	else
		ret$method = "Robust cov - estimation based on PCs (grid mode)"

	class (ret) <- "covPC"

	return (ret)
}

covPCAproj <- function (x, control)
{
	pcs = PCAproj (x, k = ncol(x), control = control)

	ret = list()
	ret$cov = pcs$loadings %*% diag (pcs$sdev^2) %*% t(pcs$loadings)
	ret$center = pcs$center

	if (!missing (control) && !is.null (control$method))
		ret$method = paste ("Robust cov - estimation based on PCs (projection mode - ", control$method, ")", sep = "")
	else
		ret$method = "Robust cov - estimation based on PCs (projection mode)"

	class (ret) <- "covPC"

	return (ret)
}

.getVtext <- function (idx)
{

	stopifnot (length (idx) == 1)

	if (idx == 1)			#	the package name
		return ("pcaPP")
	if (idx == 2)			#	the package version
		return (installed.packages ()[.getVtext (1),"Version"])
	if (idx == 3)			#	the matlab functions
		return (c ("l1median\\_HoCr", "l1median\\_VaZh", "PCAgrid", "PCAproj", "qn", "sPCAgrid"))
	if (idx == 4)			#	the example
	{
		fn <- system.file ("doc", "matlab.example.txt", package = .getVtext (1))
		return (paste (readLines (fn), collapse = "\n"))
	}

	stop ("unkown idx value")
}

ScaleAdv <- function (x, center = mean, scale = sd)
{
	if (!is.matrix (x))
	{
		if (is.data.frame (x))
			x = as.matrix(x)
		else
			x = matrix (x, ncol = 1)
	}
	n = nrow (x)
	p = ncol (x)

	m = array (0, p)

	if (missing (scale))	## 2be removed as soon as the matrix-warning message in "sd" disappears.
		scale <- .colSds

	if (is.character (center))
		center = eval (parse (text = center))

	if (is.function (center))
	{
		m = center (x)
		
		if (is.list (m))
			m <- m$par
		else if (length (m) != p)
			m = apply (x, 2, center)
		x = x - matrix(1, nrow = n) %*% m
	}
	else if (length (center) == p & is.numeric (center))
	{
		m = center
		x = x - matrix(1, nrow = n) %*% m
	}
	else if (!is.null (center))
		warning ("Unknown center specification! Centering will be omitted.\n")

	s = array (1, p)
	if (is.character (scale))
		scale = eval (parse (text = scale))

	if (is.function (scale))
	{
		s = scale (x)
		if (length (s) != p)
			s = apply (x, 2, scale)
		x = x / matrix(1, nrow = n) %*% s
	}
	else if (length (scale) == p & is.numeric (scale))
	{
		s = scale
		x = x / matrix(1, nrow = n) %*% s
	}
	else if (!is.null (scale))
		warning ("Unknown scale specification! Scaling will be omitted.\n")

	return (list (x = x, center = m, scale = s))
}

\documentclass[12pt]{article}
\usepackage{Sweave}

%\VignetteIndexEntry{Compiling pcaPP for Matlab}
%\VignetteDepends{pcaPP}
%\VignetteKeywords{Matlab}
%\VignettePackage{pcaPP}

<<results=tex,fig=FALSE,echo=FALSE>>=

source ("load.package.name.R")

library (package.name, character.only = TRUE)

vt <- eval (parse (text = paste (package.name, ":::", ".getVtext", sep = "")))

cat (sep = "",
#	"%\\VignetteIndexEntry{Compiling ", vt (1), " for Matlab}\n",	##	these lines cannot be created automatically - unfortunately..
#	"%\\VignetteDepends{", vt (1), "}\n",
#	"%\\VignetteKeywords{Matlab}\n",
#	"%\\VignettePackage{", vt (1), "}\n",
#	"\n",
	"\n",
	"\\newcommand{\\dapck}{", vt (1), "}\n",
	"\\newcommand{\\daver}{", vt (2), "}\n",
	"\n",
	"\n"
)
@

\newcommand{\sourcefile}{{\dapck}\_{\daver}.tar.gz}

\newcommand{\proglang}[1]{\textbf{#1}}
\newcommand{\code}[1]{\texttt{#1}}
\newcommand{\link}[1]{\texttt{#1}}
\newcommand{\path}[1]{{\it #1}}


\title{Compiling {\dapck} for Matlab}
\author{Heinrich Fritz}


\begin{document}
\maketitle

\section{Introduction}
The main functions of the 
\proglang{R}-package {\dapck} are implemented in an environ-mentindependent 
manner, which allows the user to use this package beyond the scope of \proglang{R}.
The package has also been prepared to be compiled and used with \proglang{Matlab}, which 
is summarized and demonstrated in this document.
The following items are required for using {\dapck} together with \proglang{Matlab}:

\begin{itemize}
\item The {\dapck} package sources \code{\sourcefile} \\(available at \link{http://CRAN.R-project.org/package=\dapck}).
\item \proglang{Matlab} (version $\geq$ 2010a).
\item A compatible \proglang{C++} compiler (for currently supported compilers see \link{http://www.mathworks.com/support/compilers/current\_release/}).
\end{itemize}
Section \ref{sec:instcomp} helps to set up a suitable compiler together with 
\proglang{Matlab}, whereas Section \ref{sec:comp} gives instructions on how to 
actually compile the package. 
Section \ref{sec:ex} demonstrates some examples on the usage of the package and 
Section \ref{sec:concl} concludes.

\section{Setting up the Compiler} \label{sec:instcomp}
Assuming that \proglang{Matlab} has already been set up properly on the target 
system, the first step is to set up a suitable \proglang{C++} compiler, such that \proglang{Matlab} 
recognizes it.
A list of compatible compilers can be obtained by typing
\begin{Scode}
>> mex -setup
   n
\end{Scode}
into the \proglang{Matlab} console.
Once a compiler from this list has been installed on the system, select it
(by using the previous command)
and make sure that \proglang{Matlab} locates it correctly.
Note that after installing a compiler \proglang{Matlab} might have to be 
restarted for correctly recognizing it.
Finally assure that the compiler has been set up properly by typing
\begin{Scode}
>> mex.getCompilerConfigurations ('C++')
\end{Scode}
\proglang{Matlab} should now correctly display the chosen compiler's details.
A more extensive introduction to the mex-interface and its configuration can be found at
\code{http://www.mathworks.de/support/tech-notes/1600/1605.html}.

\section{Compiling {\dapck}} \label{sec:comp}

Extract the downloaded package sources (\code{\sourcefile}) to a working directory,
(e.g. \path{C:/work}),
and set \proglang{Matlab}'s current directory to the \path{{\dapck}/matlab} subfolder:
\begin{Scode}
<<results=tex,fig=FALSE,echo=FALSE>>=
cat (sep = "",
	">> cd ('C:/work/", vt(1), "/matlab')")
@

\end{Scode}
Now the package is ready to be compiled by calling {\dapck}'s \code{setup} routine:
\begin{Scode}
>> setup
Changing the current directory to '../src' ... ok
<<results=tex,fig=FALSE,echo=FALSE>>=
cat (sep = "",
	"Compiling the ", vt(1), " package ... ok\n",
	"Copying the '", vt(1), ".mex*' file(s) to '../matlab' ... ok\n",
	"Changing the current directory back to '../matlab' ... ok\n\n",
	"  Successfully compiled the ", vt(1), " package for Matlab!")
@

\end{Scode}
Note that this \proglang{Matlab}-setup routine has been tested with Microsoft's Visual C++ 6.0 compiler.
Other compilers supported by \proglang{Matlab} are very likely to work as well, but have not been tested in this context yet.

\section{Using {\dapck}} \label{sec:ex}
Once the preceding code has been executed successfully, the {\dapck} package can 
be used almost the same way as in \proglang{R}.
The following functions are available in \proglang{Matlab}:
<<results=tex,fig=FALSE,echo=FALSE>>=
cat (paste ("\\code{", vt(3), "}", sep = "", collapse = ", "))
@

and work as described in the \proglang{R} man pages:

\begin{Scode}
<<results=tex,fig=FALSE,echo=FALSE>>=
	cat (vt (4))
@


\end{Scode}

\section{Conclusions} \label{sec:concl}
The configuration of a \proglang{C++} compiler in the context of \proglang{Matlab} has 
been discussed briefly, as well as how to compile the \proglang{R} package {\dapck} in 
this environment.
Further some examples on how to use the package in \proglang{Matlab} were given.
Due to the package's architecture the same \proglang{C++} sources can be used in 
both environments, which increases the availability of this software beyond the scope
of the \proglang{R} community.

\end{document}

\name{objplot}
\alias{objplot}
\title{ Objective Function Plot for Sparse PCs }
\description{
Plots an objective function (TPO or BIC) of one or more sparse PCs for a 
series of lambdas.
}
\usage{
  objplot (x, k, ...)
}
\arguments{
  \item{x}{ An \code{\link{opt.TPO}} or \code{\link{opt.BIC}} object. }
  \item{k}{ This function displays the objective function's values of the 
    \code{k}-th component for \code{opt.TPO}-objects, or the first 
    \code{k} components for \code{opt.BIC}-objects.
  }
  \item{...}{ Further arguments passed to or from other functions. }
}
\details{
This function operates on the return object of function
\code{\link{opt.TPO}} or \code{\link{opt.BIC}}.
The model (\code{lambda}) selected by the minimization of the corresponding 
criterion is highlighted by a dashed vertical line.

The component the argument \code{k} refers to, corresponds to the 
\code{$pc.noord} item of argument \code{x}.
For more info on the order of sparse PCs see the details section of 
\code{\link{opt.TPO}}.
}
%\value{none }
\references{
C. Croux, P. Filzmoser, H. Fritz (2011).
Robust Sparse Principal Component Analysis Based on Projection-Pursuit,
\emph{??} To appear.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{sPCAgrid}}, \code{\link{princomp}} }
\examples{

  set.seed (0)
                      ##  generate test data
  x <- data.Zou (n = 250)

  k.max <- 3          ##  max number of considered sparse PCs

                      ##  arguments for the sPCAgrid algorithm
  maxiter <- 25       ##    the maximum number of iterations
  method <- "sd"      ##    using classical estimations

                      ##  Optimizing the TPO criterion
  oTPO <- opt.TPO (x, k.max = k.max, method = method, maxiter = maxiter)

                      ##  Optimizing the BIC criterion
  oBIC <- opt.BIC (x, k.max = k.max, method = method, maxiter = maxiter)

          ##  Objective function vs. lambda
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        objplot (oTPO, k = i)
  for (i in 1:k.max)        objplot (oBIC, k = i)
}
\keyword{multivariate}
\keyword{robust}

\name{covPCA}
\alias{covPCAproj}
\alias{covPCAgrid}
\title{ Robust Covariance Matrix Estimation }
\description{
  computes the robust covariance matrix using the \code{PCAgrid} and 
  \code{PCAproj} functions.
}
\usage{
covPCAproj(x, control)
covPCAgrid(x, control)
}
\arguments{
  \item{x}{ a numeric matrix or data frame which provides the data. }
  \item{control}{ a list whose elements must be the same as (or a subset of) 
  the parameters of the appropriate PCA function (\code{\link{PCAgrid}} or 
  \code{\link{PCAproj}}). }
}
\details{
  The functions \code{covPCAproj} and \code{covPCAgrid} use the functions
  \code{\link{PCAproj}} and \code{\link{PCAgrid}} respectively to estimate 
  the covariance matrix of the data matrix \code{x}.
}
\value{
  \item{cov}{ the actual covariance matrix estimated from \code{x}}
  \item{center}{ the center of the data \code{x} that was substracted from them
  before the PCA algorithms were run.}
  \item{method}{ a string describing the method that was used to calculate the
  covariance matrix estimation}
}
\references{ 
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis, 
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}> }
\seealso{ \code{\link{PCAgrid}}, \code{\link{ScaleAdv}}, \code{\link{princomp}} }
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  covPCAproj(x)
  # compare with classical covariance matrix:
  cov(x)
}
\keyword{ robust }
\keyword{ multivariate }

\name{plot.opt.TPO}
\alias{plot.opt.TPO}
\alias{plot.opt.BIC}
\title{ Tradeoff Curves for Sparse PCs }
\description{
Tradeoff curves of one or more sparse PCs for a series of lambdas, which 
contrast the loss of explained variance and the gain of sparseness.
}
\usage{
\method{plot}{opt.TPO} (x, k, f.x = c ("l0", "pl0", "l1", "pl1", "lambda"),
                        f.y = c ("var", "pvar"), ...)
\method{plot}{opt.BIC} (x, k, f.x = c ("l0", "pl0", "l1", "pl1", "lambda"),
                        f.y = c ("var", "pvar"), ...)
}
\arguments{
  \item{x}{ An \code{\link{opt.TPO}} or \code{\link{opt.BIC}} object. }
  \item{k}{ This function plots the tradeoff curve of the 
    \code{k}-th component for \code{opt.TPO}-objects, or the first 
    \code{k} components for \code{opt.BIC}-objects.
  }
  \item{f.x, f.y}{ A string, specifying which information shall be plotted on the 
  x and y - axis. See the details section for more information.}
  \item{...}{ Further arguments passed to or from other functions. }
}
\details{
The argument \code{f.x} can obtain the following values:
\itemize{
  \item \code{"l0"}: l0 - sparseness, which corresponds to the number of 
    zero loadings of the considered component(s).
  \item \code{"pl0"}: l0 - sparseness in percent (l0 - sparseness 
    ranges from \code{0} to \code{p-1} for each component).
  \item \code{"l1"}: l1 - sparseness, which corresponds to 
    the negative sum of absolute 
    loadings of the considered component(s).\cr
    (The exact value displayed for a single component is
    \code{sqrt (p) - S}, with \code{S} as the the absolute sum of loadings.)
    \cr
    As this value is a part of the objective function which selects 
    the candidate directions within the \code{\link{sPCAgrid}} function,
    this option is provided here.
    \item \code{"pl1"} The "l1 - sparseness" in percent (l1 - sparseness 
    ranges from \code{0} to \code{sqrt (p-1)} for each component).
  \item \code{"lambda"}: The lambda used for computing a particular model.
}
The argument \code{f.y} can obtain the following values:
\itemize{
  \item \code{"var"}: The (cumulated) explained variance of the considered
  component(s). The value shown here is calculated using the variance 
  estimator specified via the \code{method} argument of function 
  \code{\link{sPCAgrid}}.
  \item \code{"pvar"}: The (cumulated) explained variance of the considered
  component(s) in percent. The 100\%-level is assumed as the sum of variances 
  of all columns of argument \code{x}.\cr
  Again the same variance estimator is 
  used as specified via the \code{method} argument of function 
  \code{\link{sPCAgrid}}.\cr
}

The subtitle summarizes the result of the applied criterion for selecting a
value of lambda:
\itemize{
\item The name of the applied method (TPO/BIC).
\item The selected value of \code{lambda} for the \code{k}-th component 
  (\code{\link{opt.TPO}}) or all computed components (\code{\link{opt.BIC}}).
\item The empirical cumulated variance (ECV) of the first \code{k} components 
  in percent.
\item The obtained l0-sparseness of the first \code{k} components.
}

This function operates on the return object of function
\code{\link{opt.TPO}} or \code{\link{opt.BIC}}.
The model (\code{lambda}) selected by the minimization of the corresponding 
criterion is highlighted by a dashed vertical line.

The component the argument \code{k} refers to, corresponds to the 
\code{$pc.noord} item of argument \code{x}.
For more info on the order of sparse PCs see the details section of 
\code{\link{opt.TPO}}.
}
%\value{none }
\references{
C. Croux, P. Filzmoser, H. Fritz (2011).
Robust Sparse Principal Component Analysis Based on Projection-Pursuit,
\emph{??} To appear.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{sPCAgrid}}, \code{\link{princomp}} }
\examples{

  set.seed (0)
                      ##  generate test data
  x <- data.Zou (n = 250)

  k.max <- 3          ##  max number of considered sparse PCs

                      ##  arguments for the sPCAgrid algorithm
  maxiter <- 25       ##    the maximum number of iterations
  method <- "sd"      ##    using classical estimations

                      ##  Optimizing the TPO criterion
  oTPO <- opt.TPO (x, k.max = k.max, method = method, maxiter = maxiter)

                      ##  Optimizing the BIC criterion
  oBIC <- opt.BIC (x, k.max = k.max, method = method, maxiter = maxiter)

          ##  Tradeoff Curves: Explained Variance vs. sparseness
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        plot (oTPO, k = i)
  for (i in 1:k.max)        plot (oBIC, k = i)

          ##  Explained Variance vs. lambda
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        plot (oTPO, k = i, f.x = "lambda")
  for (i in 1:k.max)        plot (oBIC, k = i, f.x = "lambda")
}
\keyword{multivariate}
\keyword{robust}

\name{PCAgrid}
\alias{PCAgrid}
\alias{sPCAgrid}
\title{ (Sparse) Robust Principal Components using the Grid search algorithm }
\description{
  Computes a desired number of (sparse) (robust) principal components using 
  the grid search algorithm in the plane.
  The global optimum of the objective function is searched in planes, not in
  the p-dimensional space, using regular grids in these planes.
}
\usage{
PCAgrid (x, k = 2, method = c ("mad", "sd", "qn"),
         maxiter = 10, splitcircle = 25, scores = TRUE, zero.tol = 1e-16, 
	 center = l1median, scale, trace = 0, store.call = TRUE, control, ...)

sPCAgrid (x, k = 2, method = c ("mad", "sd", "qn"), lambda = 1,
          maxiter = 10, splitcircle = 25, scores = TRUE, zero.tol = 1e-16, 
	  center = l1median, scale, trace = 0, store.call = TRUE, control, ...)
}
\arguments{
  \item{x}{ a numerical matrix or data frame of dimension (\code{n x p})which 
  provides the data for the principal components analysis. }
  \item{k}{ the desired number of components to compute }
  \item{method}{ the scale estimator used to detect the direction with the 
  largest variance. Possible values are \code{"sd"}, \code{"mad"} and 
  \code{"qn"}, the  latter can be called \code{"Qn"} too. \code{"mad"} is the 
  default value.}
  \item{lambda}{ the sparseness constraint's strength(\code{sPCAgrid} only).
    A single value for all components, or a vector of length \code{k} with 
    different values for each component can be specified.
    See \code{\link{opt.TPO}} for the choice of this argument. }
  \item{maxiter}{ the maximum number of iterations. }
  \item{splitcircle}{ the number of directions in which the algorithm should
  search for the largest variance. The direction with the largest variance
  is searched for in the directions defined by a number of equally spaced points
  on the unit circle. This argument determines, how many such points are used to
  split the unit circle. }
  \item{scores}{ A logical value indicating whether the scores of the
  principal component should be calculated. }
  \item{zero.tol}{ the zero tolerance used internally for checking 
  convergence, etc. }
%  \item{anglehalving}{ boolean stating whether angle halving is to be used or not. 
%  Angle halving will usually improve the solution quite a lot.}
%  \item{fact2dim}{ an integer that is multiplied to splitcircle if x is only 
%  two-dimensional. In higher dimensions, fewer search directions are needed to allow
%  for faster computation. In two dimensions, more search directions are required to 
%  grant higher precision. \code{fact2dim} is used to take account of this.}
  \item{center}{ this argument indicates how the data is to be centered. It
  can be a function like \code{\link{mean}} or \code{\link{median}} or a vector
  of length \code{ncol(x)} containing the center value of each column. }
  \item{scale}{ this argument indicates how the data is to be rescaled. It
  can be a function like \code{\link{sd}} or \code{\link{mad}} or a vector
  of length \code{ncol(x)} containing the scale value of each column. }
  \item{trace}{ an integer value >= 0, specifying the tracing level. }
%  \item{cut.pc}{ a logical value, specifying whether only the first \code{k} 
%  columns of the resulting loadings and scores matrix shall be returned 
%  (\code{TRUE}). If this value is \cpode{FALSE}, the algorithm returns 
%  an \code{p x p} loadings- and an \code{n x p} scores matrix, whereas the
%  last \code{p - k} components form an arbitrary basis of the complementary
%  space of the first \code{k} found components.}
%  \item{pc.ini}{ an optional pre calculated \code{princomp} (S3) object. If
%  provided, the algorithm searches for additional PCs in the orthogonal space
%  of the components provided by this object. }
%  \item{k.ini}{ an optional integer value, specifying how many components of
%  \code{pc.ini} shall be considered. }
%  \item{ord.all}{ a logical value, specifying wheter}
%  \item{HDred}{ }
  \item{store.call}{ a logical variable, specifying whether the function call
    shall be stored in the result structure. }
  \item{control}{ a list which elements must be the same as (or a subset of) 
  the parameters above. If the control object is supplied, the parameters from 
  it will be used and any other given parameters are overridden. }
  \item{...}{ further arguments passed to or from other functions. }
}
\details{
  In contrast to \code{PCAgrid}, the function \code{sPCAgrid} computes sparse
  principal components. The strength of the applied sparseness constraint is
  specified by argument \code{lambda}.
  %Setting \code{lambda = 0} yields the same estimation for both functions 
  %\code{PCAgrid} and\code{sPCAgrid}.

  Similar to the function \code{\link{princomp}}, there is a \code{print} method 
  for the these objects that prints the results in a nice format and the 
  \code{plot} method produces a scree plot (\code{\link{screeplot}}). There is 
  also a \code{biplot} method.

  Angle halving is an extension of the original algorithm. In the original 
  algorithm, the search directions are determined by a number of points on the
  unit circle in the interval [-pi/2 ; pi/2). Angle halving means this angle is
  halved in each iteration, eg. for the first approximation, the above mentioned
  angle is used, for the second approximation, the angle is halved to 
  [-pi/4 ; pi/4) and so on. This usually gives better results with less 
  iterations needed. \cr
  NOTE: in previous implementations angle halving could be suppressed by the 
  former argument "\code{anglehalving}". This still can be done by setting 
  argument \code{maxiter = 0}.
}
\value{
  The function returns an object of class \code{"princomp"}, i.e. a list 
  similar to the output of the function \code{\link{princomp}}.
  \item{sdev}{the (robust) standard deviations of the principal components.}
  \item{loadings}{the matrix of variable loadings (i.e., a matrix whose columns
  contain the eigenvectors).  This is of class \code{"loadings"}:
  see \code{\link{loadings}} for its \code{\link{print}} method.}
  \item{center}{the means that were subtracted.}
  \item{scale}{the scalings applied to each variable.}
  \item{n.obs}{the number of observations.}
  \item{scores}{if \code{scores = TRUE}, the scores of the supplied data on the
  principal components.}
  \item{call}{the matched call.}
  \item{obj}{A vector containing the objective functions values. For function
  \code{PCAgrid} this is the same as \code{sdev}. }
  \item{lambda}{The lambda each component has been calculated with 
  (\code{\link{sPCAgrid}} only).}
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.

  C. Croux, P. Filzmoser, H. Fritz (2011).
  Robust Sparse Principal Component Analysis Based on Projection-Pursuit,
  \emph{??} To appear.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{PCAproj}}, \code{\link{princomp}} }
\note{See the vignette "Compiling pcaPP for Matlab" which comes with this package to compile and use these functions in Matlab.}
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  # Here we calculate the principal components with PCAgrid
  pc <- PCAgrid(x)
  # we could draw a biplot too:
  biplot(pc)
  # now we want to compare the results with the non-robust principal components
  pc <- princomp(x)
  # again, a biplot for comparison:
  biplot(pc)

  ##  Sparse loadings
  set.seed (0)
  x <- data.Zou ()

                   ##  applying PCA
  pc <-  princomp (x)
                   ##  the corresponding non-sparse loadings
  unclass (pc$load[,1:3])
  pc$sdev[1:3]

                   ##  lambda as calculated in the opt.TPO - example
  lambda <- c (0.23, 0.34, 0.005)
                   ##  applying sparse PCA
  spc <- sPCAgrid (x, k = 3, lambda = lambda, method = "sd")
  unclass (spc$load)
  spc$sdev[1:3]

                   ## comparing the non-sparse and sparse biplot
  par (mfrow = 1:2)
  biplot (pc, main = "non-sparse PCs")
  biplot (spc, main = "sparse PCs")
}
\keyword{multivariate}
\keyword{robust}

\name{cor.fk}
\alias{cor.fk}
\title{ Fast estimation of Kendall's tau rank correlation coefficient}
\description{
  Calculates Kendall's tau rank correlation coefficient in O (n log (n)) 
  rather than O (n\^2) as in the current R implementation.
}
\usage{
%cor.fk (x, y = NULL, cor = TRUE)
cor.fk (x, y = NULL)
}
\arguments{
  \item{x}{ A vector, a matrix or a data frame of data. }
  \item{y}{ A vector of data. }
%  \item{cor}{ A logical value, specifying whether the correlation (\code{TRUE}) or the covariance (\code{FALSE} shall be estimated. )}
}
\value{
  The estimated correlation coefficient.
}
\details{
The code of this implementation of the fast Kendall's tau correlation algorithm has 
originally been published by David Simcha. 
Due to it's runtime (\code{O(n log n)}) it's essentially faster than the 
current R implementation (\code{O (n\^2)}), especially for large numbers of 
observations.
The algorithm goes back to Knight (1966) and has been described more detailed 
by Abrevaya (1999) and Christensen (2005).
}
\references{ 
  Knight, W. R. (1966). A Computer Method for Calculating Kendall's Tau with Ungrouped Data.
    Journal of the American Statistical Association, \strong{314}(61) Part 1, 436-439.\cr
  Christensen D. (2005). Fast algorithms for the calculation of Kendall's Tau.
    Journal of Computational Statistics \strong{20}, 51-62.\cr
  Abrevaya J. (1999). Computation of the Maximum Rank Correlation Estimator.
    Economic Letters \strong{62}, 279-285.
}
\author{ David Simcha, Heinrich Fritz, Christophe Croux, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}

\seealso{ \code{\link{cor}} }
\examples{

  set.seed (100)		## creating test data
  n <- 1000
  x <- rnorm (n)
  y <- x+  rnorm (n)

  tim <- proc.time ()[1]	## applying cor.fk
  cor.fk (x, y)
  cat ("cor.fk runtime [s]:", proc.time ()[1] - tim, "(n =", length (x), ")\n")

  tim <- proc.time ()[1]	## applying cor (standard R implementation)
  cor (x, y, method = "kendall")
  cat ("cor runtime [s]:", proc.time ()[1] - tim, "(n =", length (x), ")\n")

		##	applying cor and cor.fk on data containing				

  Xt <- cbind (c (x, as.integer (x)), c (y, as.integer (y)))

  tim <- proc.time ()[1]	## applying cor.fk
  cor.fk (Xt)
  cat ("cor.fk runtime [s]:", proc.time ()[1] - tim, "(n =", nrow (Xt), ")\n")

  tim <- proc.time ()[1]	## applying cor (standard R implementation)
  cor (Xt, method = "kendall")
  cat ("cor runtime [s]:", proc.time ()[1] - tim, "(n =", nrow (Xt), ")\n")
}
\keyword{multivariate}
\keyword{robust}

\name{qn}
\alias{qn}
\title{ scale estimation using the robust Qn estimator }
\description{
  Returns a scale estimation as calculated by the (robust) Qn estimator.
}
\usage{
qn(x, corrFact)
}
\arguments{
  \item{x}{ a vector of data }
  \item{corrFact}{ the finite sample bias correction factor. By default a value of ~ 2.219144 is used (assuming normality). }
}
\details{
  The Qn estimator computes the first quartile of the pairwise absolute 
  differences of all data values.
}
\value{
  The estimated scale of the data.
}
\references{ 
  P.J. Rousseeuw, C. Croux (1993)
  Alternatives to the Median Absolute Deviation,
  \emph{JASA}, \strong{88}, 1273-1283.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}

\section{Warning }{
Earlier implementations used a wrong correction factor for the final result. Thus qn estimations computed with package pcaPP version > 1.8-1 differ about 0.12\% from earlier estimations (version <= 1.8-1).
}

\seealso{ \code{\link{mad}} }
\note{See the vignette "Compiling pcaPP for Matlab" which comes with this package to compile and use this function in Matlab.}
\examples{
  # data with outliers
  x <- c(rnorm(100), rnorm(10, 10))
  qn(x)
}
\keyword{multivariate}
\keyword{robust}

\name{PCAproj}
\alias{PCAproj}
\title{ Robust Principal Components using the algorithm of Croux and Ruiz-Gazen (2005) }
\description{
  Computes a desired number of (robust) principal components using the algorithm of 
  Croux and Ruiz-Gazen (JMVA, 2005). 
}
\usage{
PCAproj(x, k = 2, method = c("mad", "sd", "qn"), CalcMethod = c("eachobs",
"lincomb", "sphere"), nmax = 1000, update = TRUE, scores = TRUE, maxit = 5, 
maxhalf = 5, scale = NULL, center = l1median_NLM, zero.tol = 1e-16, control)
}
\arguments{
  \item{x}{ a numeric matrix or data frame which provides the data for the 
  principal components analysis. }
  \item{k}{ desired number of components to compute }
  \item{method}{ scale estimator used to detect the direction with the largest 
  variance. Possible values are \code{"sd"}, \code{"mad"} and \code{"qn"}, the 
  latter can be called \code{"Qn"} too. \code{"mad"} is the default value.}
  \item{CalcMethod}{ the variant of the algorithm to be used. Possible values are
  \code{"eachobs"}, \code{"lincomb"} and \code{"sphere"}, with \code{"eachobs"} being 
  the default. }
  \item{nmax}{ maximum number of directions to search in each step (only when
  using \code{"sphere"} or \code{"lincomb"} as the \code{CalcMethod}). }
  \item{update}{ a logical value indicating whether an update algorithm should be 
  used.}
  \item{scores}{ a logical value indicating whether the scores of the
  principal component should be calculated. }
  \item{maxit}{ maximim number of iterations. }
  \item{maxhalf}{ maximum number of steps for angle halving. }
  \item{scale}{ this argument indicates how the data is to be rescaled. It
  can be a function like \code{\link{sd}} or \code{\link{mad}} or a vector
  of length \code{ncol(x)} containing the scale value of each column. }
  \item{center}{ this argument indicates how the data is to be centered. It
  can be a function like \code{\link{mean}} or \code{\link{median}} or a vector
  of length \code{ncol(x)} containing the center value of each column. }
  \item{zero.tol}{ the zero tolerance used internally for checking 
  convergence, etc. }
  \item{control}{ a list which elements must be the same as (or a subset of) 
  the parameters above. If the control object is supplied, the parameters from 
  it will be used and any other given parameters are overridden. }
}
\details{
  Basically, this algrithm considers the directions of each observation 
  through the origin of the centered data as possible projection directions.
  As this algorithm has some drawbacks, especially if \code{ncol(x) > nrow(x)}
  in the data matrix, there are several improvements that can be used with this
  algorithm.
\itemize{
  \item{update}{An updating step basing on the algorithm for finding the
  eigenvectors is added to the algorithm. This can be used with any 
  \code{CalcMethod} }
  \item{sphere}{Additional search directions are added using random directions.
  The random directions are determined using random data points generated from 
  a p-dimensional multivariate standard normal distribution. These new data 
  points are projected to the unit sphere, giving the new search directions.}
  \item{lincomb}{Additional search directions are added using linear 
  combinations of the observations. It is similar to the 
  \code{"sphere"}-algorithm, but the new data points are generated using linear 
  combinations of the original data \code{b_1*x_1 + ... + b_n*x_n} where the 
  coefficients \code{b_i} come from a uniform distribution in the interval 
  \code{[0, 1]}. }
}
  Similar to the function \code{\link{princomp}}, there is a \code{print} method 
  for the these objects that prints the results in a nice format and the \code{plot} 
  method produces a scree plot (\code{\link{screeplot}}). There is also a 
  \code{\link{biplot}} method.
}
\value{
  The function returns a list of class \code{"princomp"}, i.e. a list similar to the 
  output of the function \code{\link{princomp}}.
  \item{sdev}{the (robust) standard deviations of the principal components.}
  \item{loadings}{the matrix of variable loadings (i.e., a matrix whose columns
  contain the eigenvectors).  This is of class \code{"loadings"}:
  see \code{\link{loadings}} for its \code{\link{print}} method.}
  \item{center}{the means that were subtracted.}
  \item{scale}{the scalings applied to each variable.}
  \item{n.obs}{the number of observations.}
  \item{scores}{if \code{scores = TRUE}, the scores of the supplied data on the
  principal components.}
  \item{call}{the matched call.}
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{PCAgrid}}, \code{\link{ScaleAdv}}, \code{\link{princomp}} }
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  # Here we calculate the principal components with PCAgrid
  pc <- PCAproj(x, 6)
  # we could draw a biplot too:
  biplot(pc)

  # we could use another calculation method and another objective function, and 
  # maybe only calculate the first three principal components:
  pc <- PCAproj(x, 3, "qn", "sphere")
  biplot(pc)

  # now we want to compare the results with the non-robust principal components
  pc <- princomp(x)
  # again, a biplot for comparision:
  biplot(pc)
}
\keyword{ robust }
\keyword{ multivariate }

\name{plotcov}
\alias{plotcov}
\title{ Compare two Covariance Matrices in Plots}
\description{
  allows a direct comparison of two estimations of the covariance 
  matrix (e.g. resulting from covPC) in a plot.
}
\usage{
plotcov(cov1, cov2, method1, labels1, method2, labels2, ndigits, ...)
}
\arguments{
  \item{cov1}{ a covariance matrix (from cov, covMcd, covPC, 
     covPCAgrid, covPCAproj, etc. }
  \item{cov2}{ a covariance matrix (from cov, covMcd, covPC, 
     covPCAgrid, covPCAproj, etc. }
  \item{method1}{ legend for ellipses of estimation method1}
  \item{method2}{ legend for ellipses of estimation method2}
  \item{labels1}{ legend for numbers of estimation method1}
  \item{labels2}{ legend for numbers of estimation method2}
  \item{ndigits}{ number of digits to use for printing covariances, by default 
    ndigits=4}
  \item{...}{ additional arguments for text or plot}
}
\details{
  Since (robust) PCA can be used to re-compute the (robust) covariance matrix,
  one might be interested to compare two different methods of covariance
  estimation visually. This routine takes as input objects for the covariances
  to compare the output of \code{\link{cov}}, but also the return objects 
  from \code{\link{covPCAgrid}}, \code{\link{covPCAproj}}, \code{\link{covPC}},
  and \code{\link[robustbase]{covMcd}}. The comparison of the two covariance matrices
  is done by numbers (the covariances) and by ellipses.
}
\value{
  only the plot is generated
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}> }
\seealso{ \code{\link{PCAgrid}}, \code{\link{PCAproj}}, \code{\link{princomp}} }
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  plotcov(covPCAproj(x),covPCAgrid(x))

}
\keyword{ multivariate }

\name{l1median_NLM}
\alias{l1median_NM}
\alias{l1median_CG}
\alias{l1median_BFGS}
%\alias{l1median_SA}
\alias{l1median_NLM}
\alias{l1median_HoCr}
\alias{l1median_VaZh}
\title{ Multivariate L1 Median }
\description{
  Computes the multivariate L1 median (also called spatial median) of a data matrix \code{X}.  
}
\usage{
l1median_NM (X, maxit = 200, tol = 10^-8, trace = 0,
             m.init = .colMedians (X), ...)
l1median_CG (X, maxit = 200, tol = 10^-8, trace = 0,
             m.init = .colMedians (X), ...)%, type = 1)
l1median_BFGS (X, maxit = 200, tol = 10^-8, trace = 0,
               m.init = .colMedians (X), REPORT = 10, ...)
l1median_NLM (X, maxit = 200, tol = 10^-8, trace = 0,
              m.init = .colMedians (X), ...)
l1median_HoCr (X, maxit = 200, tol = 10^-8, zero.tol = 1e-15, trace = 0,
               m.init = .colMedians (X), ...)
l1median_VaZh (X, maxit = 200, tol = 10^-8, zero.tol = 1e-15, trace = 0,
	       m.init = .colMedians (X), ...)
}
\arguments{
  \item{X}{ a matrix of dimension \code{n} x \code{p}.}
  \item{maxit}{ The maximum number of iterations to be performed. }
  \item{tol}{ The convergence tolerance. }
  \item{trace}{ The tracing level. Set \code{trace > 0} to retrieve additional information on the single iterations. }
  \item{m.init}{ A vector of length \code{p} containing the initial value of the L1-median. }
%  \item{pscale}{ A vector of length \code{p} containing the variables scale to be used. }
  \item{REPORT}{ A parameter internally passed to \code{\link[stats]{optim}}. }
  \item{zero.tol}{ The zero-tolerance level used in \code{l1median_VaZh} and \code{l1median_HoCr} for determining the equality of two observations (i.e. an observation and a current center estimate). }
  \item{\dots}{ Further parameters passed from other functions. }
%  \item{TMax}{  }
%  \item{TempInit}{  }
}
\value{
%  returns the vector of the coordinates of the L1 median.
%  return (list (par = ret$med, value = ret$dpar.out[1], code = ret$npar.out [1], iterations = ret$npar.out [2], iterations_gr = ret$npar.out [3]))
  \item{par}{ A vector of length \code{p} containing the L1-median. }
  \item{value}{ The value of the objective function \code{||X - l1median||} which is minimized for finding the L1-median. }
  \item{code}{ The return code of the optimization algorithm. See \code{\link[stats]{optim}} and \code{\link[stats]{nlm}} for further information. }
  \item{iterations}{ The number of iterations performed. }
  \item{iterations_gr}{ When using a gradient function this value holds the number of times the gradient had to be computed. }
  \item{time}{The algorithms runtime in milliseconds.}
}
\details{The L1-median is computed using the built-in functions \code{\link[stats]{optim}} and \code{\link[stats]{nlm}}. These functions are a transcript of the \code{L1median} method of package \code{robustX} (available at ftp://stat.ethz.ch/U/maechler/R/), porting as much code as possible into C++.}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}

\seealso{ \code{\link{median}}}
\note{See the vignette "Compiling pcaPP for Matlab" which comes with this package to compile and use some of these functions in Matlab.}
\examples{

  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 4), diag(c(1, 1, 2, 2))), 
             rmvnorm( 50, rep(3, 4), diag(rep(2, 4))))

  l1median_NM (x)$par
  l1median_CG (x)$par
  l1median_BFGS (x)$par
  l1median_NLM (x)$par
  l1median_HoCr (x)$par
  l1median_VaZh (x)$par

  # compare with coordinate-wise median:
  apply(x,2,median)
}
\keyword{ multivariate }
\keyword{ robust }

\name{opt.TPO}
\alias{opt.TPO}
\alias{opt.BIC}
\title{ Model Selection for Sparse (Robust) Principal Components }
\description{
  These functions compute a suggestion for the sparseness parameter 
  \code{lambda} which is required by function \code{\link{sPCAgrid}}.
  A range of different values for lambda is tested and 
  according to an objective function, the best solution is selected.
  Two different approaches (TPO and BIC) are available, which is further 
  discussed in the details section.
  A graphical summary of the optimization can be obtained by plotting 
  the function's return value (\code{\link{plot.opt.TPO}}, 
  \code{\link{plot.opt.BIC}} for tradeoff curves or \code{\link{objplot}}
  for an objective function plot).
}
\usage{
  opt.TPO (x, k.max = ncol (x), n.lambda = 30, lambda.max, ...)
  opt.BIC (x, k.max = ncol (x), n.lambda = 30, lambda.max, ...)
}
\arguments{
  \item{x}{ a numerical matrix or data frame of dimension (\code{n x p}), 
  which provides the data for the principal components analysis. }
  \item{k.max}{ the maximum number of components which shall be considered
  for optimizing an objective function (optional). }
  \item{n.lambda}{ the number of lambdas to be checked for each component 
  (optional). }
  \item{lambda.max}{ the maximum value of lambda to be checked (optional).
  If omitted, the lambda which yields "full sparseness" (i.e. loadings of 
  only zeros and ones) is computed and used as default value. }
  \item{...}{ further arguments passed to \code{\link{sPCAgrid}} }
}
\details{
% explain difference between TPO and BIC
The choice for a particular lambda is done by optimizing an objective function, 
which is calculated for a set of \code{n.lambda} models with different 
lambdas, ranging from 0 to \code{lambda.max}. If \code{lambda.max} is not 
specified, the minimum lambda yielding "full sparseness" is used.
"Full sparseness" refers to a model with minimum possible absolute sum of 
loadings, which in general implies only zeros and ones in the loadings matrix.

The user can choose between two optimization methods: 
TPO (Tradeoff Product Optimization; see below), or the 
BIC (see Guo et al., 2011; Croux et al., 2011).
The main difference is, that optimization based on the BIC always chooses the 
same lambda for all PCs, and refers to a particular choice of \code{k}, 
the number of considered components.
TPO however is optimized separately for each component, and so delivers 
different lambdas within a model and does not depend on a decision on \code{k}.
\cr
This characteristic can be noticed in the return value of the function:
\code{\link{opt.TPO}} returns a single model with \code{k.max} PCs and 
different values of \code{lambda} for each PC. 
On the contrary \code{\link{opt.BIC}} returns \code{k.max} distinct models 
with \code{k.max} different lambdas, whereas for each model a different
number of components \code{k} has been considered for the optimization.
Applying the latter method, the user finally has to select one of these 
\code{k.max} models manually, 
e.g. by considering the cumulated explained variance,
whereas the TPO method does not require any further decisions.

% TPO 
The tradeoff made in the context of sparse PCA refers to the loss of explained
variance vs. the gain of sparseness. TPO (Tradeoff Product Optimization) 
maximizes the area under the tradeoff curve (see \code{\link{plot.opt.TPO}}),
in particular it maximizes the explained variance multiplied by the number of 
zero loadings of a particular component. As in this context the according 
criterion is minimized, the negative product is considered.

% explain problem of ordered / not ordered PCs
Note that in the context of sparse PCA, there are two sorting orders of PCs, 
which must be considered: Either according to the objective function's value, 
(item \code{$pc.noord})or the variance of each PC(item \code{$pc}).
As in none-sparse PCA the objective function is identical to the PCs' 
variance, this is not an issue there.\cr
The sPCAgrid algorithm delivers the components in decreasing order, according 
to the objective function (which apart from the variance also includes sparseness
terms), whereas the method \code{\link{sPCAgrid}} subsequently re-orders the 
components according to their explained variance.
}
\value{
The functions return an S3 object of type "opt.TPO" or "opt.BIC" respectively, 
containing the following items:
\item{pc}{
An S3 object of type \code{princomp} (\code{\link{opt.TPO}}), or a list of 
  such objects of length \code{k.max} (\code{\link{opt.BIC}}), 
  as returned by \code{\link{sPCAgrid}}.}
\item{pc.noord}{An S3 object of type \code{princomp} (\code{\link{opt.TPO}}), 
  or a list of such objects of length \code{k.max} (\code{\link{opt.BIC}}), 
  as returned by \code{\link{sPCAgrid}}.\cr
  Here the PCs have not been re-ordered according 
  to their variance, but are still ordered according to their objective 
  function's value
  as returned by the \code{\link{sPCAgrid}} - algorithm.
  This information is used for according tradeoff curves and the objective 
  function plot.}
\item{x}{ The input data matrix as provided by the user. }
\item{k.ini, opt}{ These items contain optimization information, as used in 
  functions \code{\link{plot.opt.TPO}}, \code{\link{plot.opt.BIC}} and 
  \code{\link{objplot}}.}
}
\references{
  C. Croux, P. Filzmoser, H. Fritz (2011).
  Robust Sparse Principal Component Analysis Based on Projection-Pursuit,
  \emph{??} To appear.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{sPCAgrid}}, \code{\link{princomp}} }
\examples{

  set.seed (0)
                      ##  generate test data
  x <- data.Zou (n = 250)

  k.max <- 3          ##  max number of considered sparse PCs

                      ##  arguments for the sPCAgrid algorithm
  maxiter <- 25       ##    the maximum number of iterations
  method <- "sd"      ##    using classical estimations

                      ##  Optimizing the TPO criterion
  oTPO <- opt.TPO (x, k.max = k.max, method = method, maxiter = maxiter)

  oTPO$pc             ##  the model selected by opt.TPO
  oTPO$pc$load        ##  and the according sparse loadings.

                      ##  Optimizing the BIC criterion
  oBIC <- opt.BIC (x, k.max = k.max, method = method, maxiter = maxiter)

  oBIC$pc[[1]]        ##  the first model selected by opt.BIC (k = 1)

          ##  Tradeoff Curves: Explained Variance vs. sparseness
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        plot (oTPO, k = i)
  for (i in 1:k.max)        plot (oBIC, k = i)

          ##  Tradeoff Curves: Explained Variance vs. lambda
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        plot (oTPO, k = i, f.x = "lambda")
  for (i in 1:k.max)        plot (oBIC, k = i, f.x = "lambda")

          ##  Objective function vs. lambda
  par (mfrow = c (2, k.max))
  for (i in 1:k.max)        objplot (oTPO, k = i)
  for (i in 1:k.max)        objplot (oBIC, k = i)
}
\keyword{multivariate}
\keyword{robust}

\name{data.Zou}
\alias{data.Zou}
\title{ Test Data Generation for Sparse PCA examples }
\description{
Draws a sample data set, as introduced by Zou et al. (2006).
}
\usage{
data.Zou (n = 250, p =  c(4, 4, 2), ...)
}
\arguments{
  \item{n}{ The required number of observations. }
  \item{p}{ A vector of length 3, specifying how many variables shall be 
  constructed using the three factors V1, V2 and V3. }
  \item{...}{ Further arguments passed to or from other functions. }
}
\details{
This data set has been introduced by Zou et al. (2006), and then been referred 
to several times, e.g. by Farcomeni (2009), Guo et al. (2010) and 
Croux et al. (2011).

The data set contains two latent factors V1 ~ N(0, 290) and V2 ~ N(0, 300) and 
a third mixed component V3 = -0.3 V1 + 0.925V2 + e; e ~ N(0, 1).\cr
The ten variables Xi of the original data set are constructed the following 
way:\cr
Xi = V1 + ei; i = 1, 2, 3, 4\cr
Xi = V2 + ei; i = 5, 6, 7, 8\cr
Xi = V3 + ei; i = 9, 10\cr
whereas ei ~ N(0, 1) is indepependent for i = 1 , ..., 10
}
\value{
 A matrix of dimension \code{n x sum (p)} containing the generated sample data 
 set.
 }
\references{
C. Croux, P. Filzmoser, H. Fritz (2011).
Robust Sparse Principal Component Analysis Based on Projection-Pursuit,
\emph{??} To appear.

A. Farcomeni (2009).
An exact approach to sparse principal component analysis,
\emph{Computational Statistics}, Vol. 24(4), pp. 583-604.

J. Guo, G.  James, E. Levina, F. Michailidis, and J. Zhu (2010).
Principal component analysis with sparse fused loadings,
\emph{Journal of Computational and Graphical Statistics.} To appear.

H. Zou, T. Hastie, R. Tibshirani (2006). 
Sparse principal component analysis,
\emph{Journal of Computational and Graphical Statistics}, Vol. 15(2), pp. 265-286.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{ \code{\link{sPCAgrid}}, \code{\link{princomp}} }
\examples{
                   ##  data generation
  set.seed (0)
  x <- data.Zou ()

                   ##  applying PCA
  pc <-  princomp (x)
                   ##  the corresponding non-sparse loadings
  unclass (pc$load[,1:3])
  pc$sdev[1:3]

                   ##  lambda as calculated in the opt.TPO - example
  lambda <- c (0.23, 0.34, 0.005)
                   ##  applying sparse PCA
  spc <- sPCAgrid (x, k = 3, lambda = lambda, method = "sd")
  unclass (spc$load)
  spc$sdev[1:3]

                   ## comparing the non-sparse and sparse biplot
  par (mfrow = 1:2)
  biplot (pc, main = "non-sparse PCs")
  biplot (spc, main = "sparse PCs")
}
\keyword{multivariate}
\keyword{robust}

\name{covPC}
\alias{covPC}
\title{ Covariance Matrix Estimation from princomp Object}
\description{
  computes the covariance matrix from a princomp object. 
  The number of components k can be given as input.
}
\usage{
covPC(x, k, method)
}
\arguments{
  \item{x}{ an object of class princomp. }
  \item{k}{ number of PCs to use for covariance estimation (optional). }
  \item{method}{ method how the PCs have been estimated (optional). }
}
\details{
  There are several possibilities to estimate the principal components (PCs)
  from an input data matrix, including the functions \code{\link{PCAproj}} and 
  \code{\link{PCAgrid}}. This function uses the estimated PCs to reconstruct
  the covariance matrix. Not all PCs have to be used, the number k of
  PCs (first k PCs) can be given as input to the function.
}
\value{
  \item{cov}{ the estimated covariance matrix}
  \item{center}{ the center of the data, as provided from the princomp object.}
  \item{method}{ a string describing the method that was used to calculate the
  PCs.}
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}> }
\seealso{ \code{\link{PCAgrid}}, \code{\link{PCAproj}}, \code{\link{princomp}} }
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  pc <- princomp(x)
  covPC(pc, k=2)
}
\keyword{ multivariate }

\name{ScaleAdv}
\alias{ScaleAdv}
\title{ centers and rescales data }
\description{
  Data is centered and rescaled (to have mean 0 and a standard deviation of 1).
}
\usage{
ScaleAdv(x, center = mean, scale = sd)
}
\arguments{
  \item{x}{ matrix containing the observations. If this is not a matrix, but
  a data frame, it is automatically converted into a matrix using the function
  \code{\link{as.matrix}}. In any other case, (eg. a vector) it is converted 
  into a matrix with one single column. }
  \item{center}{ this argument indicates how the data is to be centered. It 
  can be a function like \code{\link{mean}} or \code{\link{median}} or a vector
  of length \code{ncol(x)} containing the center value of each column. }
  \item{scale}{ this argument indicates how the data is to be rescaled. It 
  can be a function like \code{\link{sd}} or \code{\link{mad}} or a vector
  of length \code{ncol(x)} containing the scale value of each column. }
}
\value{
  The function returns a list containing
  \item{ x }{centered and rescaled data matrix.}
  \item{ center }{ a vector of the centers of each column x. If you add to 
  each column of \code{x} the appropriate value from \code{center}, you will obtain
  the data with the original location of the observations. }
  \item{ scale }{ a vector of the scale factors of each column x. If you multiply 
  each column of \code{x} by the appropriate value from \code{scale}, you will obtain
  the data with the original scales. }
}
\details{
  The default \code{scale} being \code{NULL} means that no rescaling is done.
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{ Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\examples{
  x <- rnorm(100, 10, 5)
  x <- ScaleAdv(x)$x

  # can be used with multivariate data too
  library(mvtnorm)
  x <- rmvnorm(100, 3:7, diag((7:3)^2))
  res <- ScaleAdv(x, center = l1median, scale = mad)
  res

  # instead of using an estimator, you could specify the center and scale yourself too
  x <- rmvnorm(100, 3:7, diag((7:3)^2))
  res <- ScaleAdv(x, 3:7, 7:3)
  res
}
\keyword{ multivariate }

\name{PCdiagplot}
\alias{PCdiagplot}
\title{Diagnostic plot for principal components}
\description{
  Computes Orthogonal Distances (OD) and Score Distances (SD) for already computed principal components
  using the projection pursuit technique.
}
\usage{
PCdiagplot(x, PCobj, crit = c(0.975, 0.99, 0.999), ksel = NULL, plot = TRUE,
           plotbw = TRUE, raw = FALSE, colgrid = "black", ...)
}
\arguments{
  \item{x}{ a numeric matrix or data frame which provides the data for the
  principal components analysis. }
  \item{PCobj}{ a PCA object resulting from \code{\link{PCAproj}} or \code{\link{PCAgrid}} }
  \item{crit}{ quantile(s) used for the critical value(s) for OD and SD }
  \item{ksel}{ range for the number of PCs to be used in the plot; if NULL all PCs provided are used }
  \item{plot}{ if TRUE a plot is generated, otherwise only the values are returned }
  \item{plotbw}{ if TRUE the plot uses gray, otherwise color representation }
  \item{raw}{ if FALSE, the distribution of the SD will be transformed to approach chisquare 
  distribution, otherwise the raw values are reported and used for plotting }
  \item{colgrid}{ the color used for the grid lines in the plot }
  \item{\dots}{ additional graphics parameters as used in \code{\link{par}} }
}
\details{
  Based on (robust) principal components, a diagnostics plot is made using Orthogonal Distance (OD)
  and Score Distance (SD). This plot can provide important information about the multivariate
  data structure.
}
\value{
  \item{ODist}{matrix with OD for each observation (rows) and each selected PC (cols) }
  \item{SDist}{matrix with SD for each observation (rows) and each selected PC (cols) }
  \item{critOD}{matrix with critical values for OD for each selected PC (rows) and each
        critical value (cols) }
  \item{critSD}{matrix with critical values for SD for each selected PC (rows) and each
        critical value (cols) }
}
\references{ 
  P. Filzmoser and H. Fritz (2007).
  Exploring high-dimensional data with robust principal components.
  In  S. Aivazian, P. Filzmoser, and Yu. Kharin, editors, Proceedings of the Eighth 
  International Conference on Computer Data Analysis and Modeling, volume 1, pp. 43-50, 
  Belarusian State University, Minsk.

  M. Hubert, P.J. Rousseeuwm, K. Vanden Branden (2005).
  ROBCA: a new approach to robust principal component analysis
  Technometrics 47, pp. 64-79.
}
\author{Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}
\seealso{\code{\link{PCAproj}}, \code{\link{PCAgrid}}}
\examples{
  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(85, rep(0, 6), diag(c(5, rep(1,5)))),
             rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))
  # Here we calculate the principal components with PCAgrid
  pcrob <- PCAgrid(x, k=6)
  resrob <- PCdiagplot(x,pcrob,plotbw=FALSE)

  # compare with classical method:
  pcclass <- PCAgrid(x, k=6, method="sd")
  resclass <- PCdiagplot(x,pcclass,plotbw=FALSE)
}
\keyword{robust}

\name{l1median}
\alias{l1median}
\title{ Multivariate L1 Median }
\description{
  Computes the multivariate L1 median (also called spatial median) of a data matrix.  
}
\usage{
l1median(X, MaxStep = 200, ItTol = 10^-8, trace = 0, m.init = .colMedians (X))
}
\arguments{
  \item{X}{ A matrix containing the values whose multivariate L1 median is to be computed. }
  \item{MaxStep}{ The maximum number of iterations. }
  \item{ItTol}{ Tolerance for convergence of the algorithm.}
  \item{trace}{ The tracing level. }
  \item{m.init}{ An initial estimate. }
}
\value{
  returns the vector of the coordinates of the L1 median.
}
\references{
  C. Croux, P. Filzmoser, M. Oliveira, (2007).
  Algorithms for Projection-Pursuit Robust Principal Component Analysis,
  \emph{Chemometrics and Intelligent Laboratory Systems}, Vol. 87, pp. 218-225.
}
\author{Heinrich Fritz, Peter Filzmoser <\email{P.Filzmoser@tuwien.ac.at}>}

\seealso{ \code{\link{median}}}
\examples{
  l1median(rnorm(100), trace = -1) # this returns the median of the sample

  # multivariate data with outliers
  library(mvtnorm)
  x <- rbind(rmvnorm(200, rep(0, 4), diag(c(1, 1, 2, 2))), 
             rmvnorm( 50, rep(3, 4), diag(rep(2, 4))))
  l1median(x, trace = -1)
  # compare with coordinate-wise median:
  apply(x,2,median)
}
\keyword{ multivariate }
\keyword{ robust }
